﻿using Kivii.Domain.Lac.Lims.Extensions;
using Kivii;
using Kivii.Lims.Entities;
using Kivii.Lac.Lims.Extensions;
using Kivii.Linq;
using Kivii.Products.Entities;
using Kivii.Standards.Entities;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using Report = Kivii.Lims.Entities.Report;
using Sample = Kivii.Lims.Entities.Sample;

namespace Kivii.Domain.Lac.Lims.Transforms
{
    #region 更新标准值前置查询
    /// <summary>
    /// 更新前调用该接口
    /// 更新样品等级、安全类别、功能等级之前，需要查询是否存在非待检测状态（BeforeTest）的项目也处于需要更新标准值的范畴之内
    /// 如果不存在，返回空，前端允许更新标准值
    /// 如果存在，前端组织提示（XX、XX项未打回，请打回后更新标准值）
    /// </summary>
    [Api(Description = "更新标准值前置")]
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class UpdateDetectionValueQualifiedPre : RestfulExecution<ReportItem>
    {
        public Guid ReportKvid { get; set; }
        public string SampleGrade { get; set; }
        public string SampleGradeSafety { get; set; }
        public string SampleGradeFunction { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            #region 数据验证
            (ReportKvid == Guid.Empty).ThrowIfTrue("未传入报告Kvid!");
            #endregion
            var rtns = new RestfulQueryResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();
            var connLims = KiviiContext.GetOpenedDbConnection<Report>();
            //获取更新前的报告信息,并比对样品等级等内容是否更新
            var report = connLims.SingleById<Report>(ReportKvid);
            if (report.SampleGrade == SampleGrade && report.SampleGradeSafety == SampleGradeSafety && report.SampleGradeFunction == SampleGradeFunction) return rtns;
            //获取非待检测状态的项目集合
            //当前报告所有项目
            var reportItemsAll= connLims.Select<ReportItem>(o => o.ReportKvid == ReportKvid);
            //非待检测状态项目
            var reportItems = reportItemsAll.Where(o=>o.Status>(int)ReportItemStatusType.BeforeTest&&(o.Type==ReportItemType.Test||o.Type==ReportItemType.TestFolder)).ToList();
            if (reportItems.IsNullOrEmpty()) return rtns;
            //获取标准值集合
            var detectionKvids = reportItems.ConvertAll(o=>o.DetectionKvid);
            var connStd = KiviiContext.GetOpenedDbConnection<DetectionValue>();
            var detectionValues = connStd.Select<DetectionValue>(o=>Sql.In(o.DetectionKvid, detectionKvids));
            if (report.SampleGrade != SampleGrade) {
                //样品等级发生变更
                foreach (var reportItem in reportItems)
                {
                    var detectionCondition=reportItem.GetDetectionValueCondition(detectionValues);
                    if (detectionCondition != "SampleGrade") continue;
                    if (reportItem.Type == ReportItemType.TestFolder)
                    {
                        if(!rtns.Results.Exists(o => o.DetectionKvid == reportItem.DetectionKvid)) rtns.Results.Add(reportItem);
                    }
                    else {
                        var testFolder = reportItemsAll.FirstOrDefault(o => o.Kvid == reportItem.ParentKvid && o.Type == ReportItemType.TestFolder);
                        if (testFolder == null || rtns.Results.Exists(o => o.DetectionKvid == testFolder.DetectionKvid)) continue;
                        rtns.Results.Add(testFolder);
                    }
                }
            }
            if (report.SampleGradeSafety != SampleGradeSafety) {
                //安全类别发生变更
                foreach (var reportItem in reportItems)
                {
                    var detectionCondition = reportItem.GetDetectionValueCondition(detectionValues);
                    if (detectionCondition != "SampleGradeSafety") continue;
                    if (reportItem.Type == ReportItemType.TestFolder)
                    {
                        if(!rtns.Results.Exists(o => o.DetectionKvid == reportItem.DetectionKvid)) rtns.Results.Add(reportItem);
                    }
                    else
                    {
                        var testFolder = reportItemsAll.FirstOrDefault(o => o.Kvid == reportItem.ParentKvid && o.Type == ReportItemType.TestFolder);
                        if (testFolder == null || rtns.Results.Exists(o => o.DetectionKvid == testFolder.DetectionKvid)) continue;
                        rtns.Results.Add(testFolder);
                    }
                }

            }
            if (report.SampleGradeFunction != SampleGradeFunction)
            {
                //功能等级发生变更
                foreach (var reportItem in reportItems)
                {
                    var detectionCondition = reportItem.GetDetectionValueCondition(detectionValues);
                    if (detectionCondition != "SampleGradeFunction") continue;
                    if (reportItem.Type == ReportItemType.TestFolder)
                    {
                        if(!rtns.Results.Exists(o => o.DetectionKvid == reportItem.DetectionKvid)) rtns.Results.Add(reportItem);
                    }
                    else
                    {
                        var testFolder = reportItemsAll.FirstOrDefault(o => o.Kvid == reportItem.ParentKvid && o.Type == ReportItemType.TestFolder);
                        if (testFolder == null || rtns.Results.Exists(o => o.DetectionKvid == testFolder.DetectionKvid)) continue;
                       rtns.Results.Add(testFolder);
                    }
                }
            }
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }
    #endregion

    #region 更新标准值(升级)
    /// <summary>
    /// 根据报告的样品等级更新检验项目的标准值
    /// 增加功能等级参数
    /// </summary>
    [Api(Description = "更新标准值(升级)")]
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class UpdateDetectionValueQualifiedUpgrade : RestfulExecution<ReportItem>
    {
        public Guid ReportKvid { get; set; }
        public string SampleGrade { get; set; }
        public string SampleGradeSafety { get; set; }
        public string SampleGradeFunction { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            #region 数据验证
            if (ReportKvid == Guid.Empty) throw new Exception("请传入ReportKvid!");
            #endregion
            var rtns = new RestfulQueryResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();

            var connLims = KiviiContext.GetOpenedDbConnection<ReportItem>();
            var report = connLims.SingleById<Report>(ReportKvid);
            if (report == null) throw new Exception("未捕获对应报告信息!");
            //获取ReportItems集合(待检测状态)
            var queryReportItems = connLims.From<ReportItem>();
            queryReportItems.Where(o => o.ReportKvid == ReportKvid && (o.Type == ReportItemType.TestFolder || o.Type == ReportItemType.Test)&&o.Status==(int)ReportItemStatusType.BeforeTest);
            var reportItems = connLims.Select(queryReportItems);
            //获取报告所涉及的检验项目集合
            var connStandards = KiviiContext.GetOpenedDbConnection<Detection>();
            var detectionsKvids = reportItems.ConvertAll(o => o.DetectionKvid);
            var queryDetections = connStandards.From<Detection>();
            queryDetections.Where(o => Sql.In(o.Kvid, detectionsKvids));
            var detections = connStandards.Select(queryDetections);
            //获取报告所涉及项目的标准值
            var detectionKvids = detections.ConvertAll(o => o.Kvid);
            var queryDetectionValues = connStandards.From<DetectionValue>();
            queryDetectionValues.Where(o => Sql.In(o.DetectionKvid, detectionKvids));
            var detectionValues = connStandards.Select(queryDetectionValues);
            IDbTransaction trans = null;//define a null tag
            trans = connLims.OpenTransaction();
            try
            {
                foreach (var reportItem in reportItems)
                {
                    var detection = detections.Where(o => o.Kvid == reportItem.DetectionKvid).FirstOrDefault();
                    if (detection == null) continue;
                    var detectionValueQualified = detection.GetQualifiedValueUpgrade(SampleGrade, SampleGradeSafety, SampleGradeFunction, detectionValues);
                    if (reportItem.DetectionValueQualified == detectionValueQualified) continue;
                    var oldValue = reportItem.DetectionValueQualified;
                    reportItem.DetectionValueQualified = detectionValueQualified;
                    reportItem.AddOnlyProperties(o => o.DetectionValueQualified);
                    //增加标准值被修改后的标记(仅在打回的情况下)
                    if (report.Status == (int)ReportStatusType.CommissionAccept && report.Metadata != null && report.Metadata.ContainsKey("CallBackTo") && report.Metadata["CallBackTo"] == "CommissionAccept")
                    {
                        //CallBackFrom:DataEntry
                        reportItem.Metadata["IsUpdateDetectionValueQualified"] = "true";
                        reportItem.AddOnlyProperties(o => o.Metadata);
                    }
                    //创建标准值日志
                    reportItem.SetDetectionValueQualifiedLog("update", connLims, oldValue, detectionValueQualified, "UpdateDetectionValueQualifiedUpgrade");
                    //20211028与杨海峰沟通决定涉及更改标准值后,判定结果清空
                    reportItem.Judgement = string.Empty;
                    reportItem.AddOnlyProperties(o => o.Judgement);
                    connLims.UpdateOnly(reportItem);
                }
                trans?.Commit();
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }

            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }
    #endregion

    /// <summary>
    /// 根据报告的样品等级更新检验项目的标准值
    /// </summary>
    //代码初审 张梦圆 20210907
    [Api(Description = "更新标准值")]
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class UpdateDetectionValueQualified : RestfulExecution<ReportItem>
    {
        public Guid ReportKvid { get; set; }
        public string SampleGrade { get; set; }
        public string SampleGradeSafety { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            #region 数据验证
            if (ReportKvid == Guid.Empty) throw new Exception("请传入ReportKvid!");
            #endregion
            var rtns = new RestfulQueryResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();

            var connLims = KiviiContext.GetOpenedDbConnection<ReportItem>();
            //获取ReportItems集合
            var queryReportItems = connLims.From<ReportItem>();
            queryReportItems.Where(o => o.ReportKvid == ReportKvid && (o.Type == ReportItemType.TestFolder || o.Type == ReportItemType.Test));
            var reportItems = connLims.Select(queryReportItems);
            (reportItems.Where(o => o.Status != (int)ReportItemStatusType.BeforeTest).ToList().Count > 0).ThrowIfTrue("禁止修改标准值!存在状态不在委托受理的项目!");
            //获取报告所涉及的检验项目集合
            var connStandards = KiviiContext.GetOpenedDbConnection<Detection>();
            var detectionsKvids = reportItems.ConvertAll(o => o.DetectionKvid);
            var queryDetections = connStandards.From<Detection>();
            queryDetections.Where(o => Sql.In(o.Kvid, detectionsKvids));
            var detections = connStandards.Select(queryDetections);
            //获取报告所涉及项目的标准值
            var detectionKvids = detections.ConvertAll(o => o.Kvid);
            var queryDetectionValues = connStandards.From<DetectionValue>();
            queryDetectionValues.Where(o => Sql.In(o.DetectionKvid, detectionKvids));
            var detectionValues = connStandards.Select(queryDetectionValues);
            IDbTransaction trans = null;//define a null tag
            trans = connLims.OpenTransaction();
            try
            {
                foreach (var reportItem in reportItems)
                {
                    if (reportItem.StatusType != ReportItemStatusType.BeforeTest) continue;
                    var detection = detections.Where(o => o.Kvid == reportItem.DetectionKvid).FirstOrDefault();
                    if (detection == null) continue;
                    var detectionValueQualified = detection.GetQualifiedValue(SampleGrade, SampleGradeSafety, detectionValues);
                    if (reportItem.DetectionValueQualified == detectionValueQualified) continue;
                    var oldValue = reportItem.DetectionValueQualified;
                    reportItem.DetectionValueQualified = detectionValueQualified;
                    reportItem.AddOnlyProperties(o => o.DetectionValueQualified);
                    //创建标准值日志
                    reportItem.SetDetectionValueQualifiedLog("update", connLims,oldValue, detectionValueQualified, "UpdateDetectionValueQualifiedUpgrade");
                    //20211028与杨海峰沟通决定涉及更改标准值后,判定结果清空
                    reportItem.Judgement = string.Empty;
                    reportItem.AddOnlyProperties(o=>o.Judgement);
                    connLims.UpdateOnly(reportItem);
                    
                }
                trans?.Commit();
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }

            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }
    /// <summary>
    /// 根据报告的协议折扣更新检验项目的折后费
    /// </summary>
    //代码初审 张梦圆 20210907
    [Api(Description = "更新检验费")]
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class UpdateAmountPlan : RestfulExecution<ReportItem>
    {
        public ReportItem ReportItem { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            #region 数据验证
            ReportItem.Kvid.ThrowIfEmpty("未找到操作对象!");
            ReportItem.AmountPlan.ThrowIfNull("未传入检验费信息!");
            #endregion
            var rtns = new RestfulQueryResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();
            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            var reportItem = conn.SingleById<ReportItem>(ReportItem.Kvid);
            reportItem.ThrowIfNull("未找到检验项目!");
            var report = conn.SingleById<Report>(reportItem.ReportKvid);
            report.ThrowIfNull("未找到对应报告!");
            (report.AmountSettlement > 0).ThrowIfTrue("报告已结算，无法更改检验费");
            reportItem.SetAmountLog(report, "UpdateAmountPlan", reportItem.AmountPlan, ReportItem.Amount, conn);

            //获取报告涉及的合同、服务相关信息
            Quotation quotation = null;
            var connProducts = KiviiContext.GetOpenedDbConnection<Product>();
            if (report.QuotationKvid != Guid.Empty) quotation = connProducts.SingleById<Quotation>(report.QuotationKvid);
            List<QuotationDetail> quotationDetails = new List<QuotationDetail>();

            if (quotation != null)
            {
                var queryQuotationDetails = connProducts.From<QuotationDetail>();
                queryQuotationDetails.Where(o => o.QuotationKvid == quotation.Kvid);
                quotationDetails = connProducts.Select(queryQuotationDetails);
            }
            var goodsKvids = new List<Guid>();
            goodsKvids.Add(reportItem.GoodsKvid);
            var queryPrices = connProducts.From<Price>();
            queryPrices.Where(o => Sql.In(o.GoodsKvid, goodsKvids));
            var prices = connProducts.Select(queryPrices);

            reportItem.AmountPlan = ReportItem.AmountPlan;
            reportItem.AddOnlyProperties(o => o.AmountPlan);
            reportItem.CalculateAmount(quotation, quotationDetails, prices);
            //var discount = report.QuotationDiscountRate;
            //reportItem.Amount = (decimal)(ReportItem.AmountPlan * discount);
            //reportItem.AddOnlyProperties(o => o.Amount);
            conn.UpdateOnly(reportItem);
            reportItem.RemoveAllOnlyProperties();
            rtns.Results.Add(reportItem);
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }

    #region 拷贝项目
    [Api(Description = "拷贝粘贴检验项目(升级)")]
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class ReportItemPasteUpgrade : RestfulExecution<ReportItem>
    {
        public Guid FromKvid { get; set; }
        public Guid ToKvid { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            #region 数据验证
            FromKvid.ThrowIfEmpty("未传入拷贝报告的信息!");
            ToKvid.ThrowIfEmpty("未传入粘贴报告的信息!");
            #endregion
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            //获取当前拷贝报告
            var reportCopy = conn.SingleById<Report>(FromKvid);
            reportCopy.ThrowIfNull("未找到拷贝报告!");
            //获取当前粘贴报告信息
            var reportPaste = conn.SingleById<Report>(ToKvid);
            reportPaste.ThrowIfNull("未找到拷贝报告!");
            //如果粘贴报告存在检验项目,禁止操作
            var queryReportItemsByReportPaste = conn.From<ReportItem>();
            queryReportItemsByReportPaste.Where(o => o.ReportKvid == ToKvid);
            var reportItemsByReportPaste = conn.Select(queryReportItemsByReportPaste);
            (reportItemsByReportPaste.Count != 0).ThrowIfTrue("待粘贴报告存在检验项目,禁止粘贴!");
            //获取当前拷贝报告对应的检验项目
            var queryReportItems = conn.From<ReportItem>();
            queryReportItems.Where(o => o.ReportKvid == FromKvid);
            var reportItems = conn.Select(queryReportItems);
            //获取当前拷贝报告对应的检验项目中涉及到的标准值信息
            var connStandards = KiviiContext.GetOpenedDbConnection<Standard>();
            var detectionKvids = reportItems.ConvertAll(o => o.DetectionKvid);
            var queryDetections = connStandards.From<Detection>();
            queryDetections.Where(o => Sql.In(o.Kvid, detectionKvids));
            var detections = connStandards.Select(queryDetections);
            //获取标准信息(涉及到标准信息变更)
            var standardKvids = detections.ConvertAll(o => o.StandardKvid).Distinct().ToList();
            var standards = connStandards.Select<Standard>(o=>Sql.In(o.Kvid, standardKvids));
            //获取报告所涉及项目的标准值
            var queryDetectionValues = connStandards.From<DetectionValue>();
            queryDetectionValues.Where(o => Sql.In(o.DetectionKvid, detectionKvids));
            var detectionValues = connStandards.Select(queryDetectionValues);

            var rtns = new RestfulQueryResponse<Report>();
            rtns.Results = new List<Report>();
            IDbTransaction trans = null;
            trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);
            try
            {
                //找到主色号下的所有项
                var reportItemRoot = reportItems.Where(o => o.SampleKvid == o.SampleRootKvid && o.Type == ReportItemType.SampleSelf).FirstOrDefault();
                //获取需要拷贝的所有项目
                var reportItemsOriginal = new List<ReportItem>();
                if (reportItemRoot == null)
                {
                    reportItemsOriginal = reportItems;
                }
                else
                {
                    //获取主色号下的所有项目
                    reportItemsOriginal = reportItems.Where(o => o.RootKvid == reportItemRoot.Kvid && o.ParentKvid != Guid.Empty).ToList();
                }
                //获取检验项目新老Kvid集合
                var reportItemOriginalKvids = reportItemsOriginal.Select(o => o.Kvid).ToList();
                var reportItemOriginalMatchKvids = new List<MatchKvids>();
                reportItemOriginalMatchKvids.Add(new MatchKvids
                {
                    OriginalKvid = Guid.Empty,
                    MatchKvid = Guid.Empty
                });
                foreach (var reportItemKvid in reportItemOriginalKvids)
                {
                    reportItemOriginalMatchKvids.Add(new MatchKvids
                    {
                        OriginalKvid = reportItemKvid,
                        MatchKvid = Guid.NewGuid()
                    });
                }
                //不包含部位,提取所有DetectionKvid不为空的数据
                //只保留主项(去重)->处理对应子项->处理剩下的Type为Test的主项
                reportItemsOriginal = reportItemsOriginal.Where(o => o.DetectionKvid != Guid.Empty).ToList();
                var reportItemsOriginalTestFolder = reportItemsOriginal.Where(o => o.Type == ReportItemType.TestFolder).ToList();
                reportItemsOriginal = reportItemsOriginal.Except(reportItemsOriginalTestFolder).ToList();
                //记录处理过的主项值
                var detectionKvidsExist = new List<Guid>();
                foreach (var reportItemOriginalTestFolder in reportItemsOriginalTestFolder)
                {
                    //如遇相同主项直接跳过
                    if (detectionKvidsExist.Contains(reportItemOriginalTestFolder.DetectionKvid))
                    {
                        reportItemsOriginal.Remove(reportItemOriginalTestFolder);
                        var reportItemsTestFolderChildren = reportItemsOriginal.Where(o => o.ParentKvid == reportItemOriginalTestFolder.Kvid).ToList();
                        reportItemsOriginal = reportItemsOriginal.Except(reportItemsTestFolderChildren).ToList();
                        continue;
                    }
                    detectionKvidsExist.Add(reportItemOriginalTestFolder.DetectionKvid);
                    //创建主项
                    var detection = detections.Where(o => o.Kvid == reportItemOriginalTestFolder.DetectionKvid).FirstOrDefault();
                    var pasteReportItemParent = reportItemOriginalTestFolder.GetPasteReportItemUpgrade(reportPaste, detection, detectionValues,standards,true);
                    pasteReportItemParent.RootKvid = reportItemOriginalTestFolder.Kvid.GetMatchKvid(reportItemOriginalMatchKvids);
                    pasteReportItemParent.ParentKvid = Guid.Empty;
                    pasteReportItemParent.Kvid = reportItemOriginalTestFolder.Kvid.GetMatchKvid(reportItemOriginalMatchKvids);
                    pasteReportItemParent.ReportKvid = reportPaste.Kvid;
                    pasteReportItemParent.SampleKvid = reportPaste.SampleKvid;
                    pasteReportItemParent.SampleRootKvid = reportPaste.SampleKvid;
                    conn.Insert(pasteReportItemParent);
                    //创建标准值日志
                    pasteReportItemParent.SetDetectionValueQualifiedLog("create", conn,null,null, "ReportItemPasteUpgrade");
                    //创建客户要求标准值日志
                    if (pasteReportItemParent.Metadata != null && pasteReportItemParent.Metadata.ContainsKey("DetectionValueCustomer") && !pasteReportItemParent.Metadata["DetectionValueCustomer"].IsNullOrEmpty()) pasteReportItemParent.SetDetectionValueQualifiedCustomerRequiredLog("create",null, conn);
                    reportItemsOriginal.Remove(reportItemOriginalTestFolder);
                    //创建子项
                    var reportItemsOriginalChildren = reportItemsOriginal.Where(o => o.ParentKvid == reportItemOriginalTestFolder.Kvid).ToList();
                    foreach (var reportItemChild in reportItemsOriginalChildren)
                    {
                        var detectionChild = detections.Where(o => o.Kvid == reportItemChild.DetectionKvid).FirstOrDefault();
                        var pasteReportItemChild = reportItemChild.GetPasteReportItemUpgrade(reportPaste,detectionChild, detectionValues,standards,true);
                        pasteReportItemChild.RootKvid = pasteReportItemParent.Kvid;
                        pasteReportItemChild.ParentKvid = pasteReportItemParent.Kvid;
                        pasteReportItemChild.Kvid = reportItemChild.Kvid.GetMatchKvid(reportItemOriginalMatchKvids);
                        pasteReportItemChild.ReportKvid = reportPaste.Kvid;
                        pasteReportItemChild.SampleKvid = reportPaste.SampleKvid;
                        pasteReportItemChild.SampleRootKvid = reportPaste.SampleKvid;
                        conn.Insert(pasteReportItemChild);
                        reportItemsOriginal.Remove(reportItemChild);
                        //创建标准值日志
                        pasteReportItemChild.SetDetectionValueQualifiedLog("create", conn, null, null, "ReportPasteUpgrade");
                        //创建客户要求标准值日志
                        if (pasteReportItemChild.Metadata != null && pasteReportItemChild.Metadata.ContainsKey("DetectionValueCustomer") && !pasteReportItemChild.Metadata["DetectionValueCustomer"].IsNullOrEmpty()) pasteReportItemChild.SetDetectionValueQualifiedCustomerRequiredLog("create",null, conn);
                    }

                }
                //处理剩余项目:Type为Test的主项
                foreach (var reportItem in reportItemsOriginal)
                {
                    //如遇相同主项直接跳过
                    if (detectionKvidsExist.Contains(reportItem.DetectionKvid)) continue;
                    detectionKvidsExist.Add(reportItem.DetectionKvid);
                    var detection = detections.Where(o => o.Kvid == reportItem.DetectionKvid).FirstOrDefault();
                    var pasteReportItem = reportItem.GetPasteReportItemUpgrade(reportPaste,detection, detectionValues, standards, true);
                    pasteReportItem.RootKvid = reportItem.Kvid.GetMatchKvid(reportItemOriginalMatchKvids);
                    pasteReportItem.ParentKvid = Guid.Empty;
                    pasteReportItem.Kvid = reportItem.Kvid.GetMatchKvid(reportItemOriginalMatchKvids);
                    pasteReportItem.ReportKvid = reportPaste.Kvid;
                    pasteReportItem.SampleKvid = reportPaste.SampleKvid;
                    pasteReportItem.SampleRootKvid = reportPaste.SampleKvid;
                    conn.Insert(pasteReportItem);
                    //创建标准值日志
                    pasteReportItem.SetDetectionValueQualifiedLog("create", conn, null, null, "ReportItemPasteUpgrade");
                    //创建客户要求标准值日志
                    if (pasteReportItem.Metadata != null && pasteReportItem.Metadata.ContainsKey("DetectionValueCustomer") && !pasteReportItem.Metadata["DetectionValueCustomer"].IsNullOrEmpty()) pasteReportItem.SetDetectionValueQualifiedCustomerRequiredLog("create",null, conn);
                }
                rtns.Results.Add(reportPaste);
                rtns.Total = rtns.Results.Count;
                trans?.Commit();
                return rtns;
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }
        }
    }

    [Api(Description = "拷贝粘贴检验项目")]
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class ReportItemPaste : RestfulExecution<ReportItem>
    {
        public Guid FromKvid { get; set; }
        public Guid ToKvid { get; set; }
        public PasteType Type { get; set; }
        public bool IsContainParts { get; set; }
        public bool IsCopyDeteValueCustomerRequired { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            #region 数据验证
            FromKvid.ThrowIfEmpty("未传入拷贝报告的信息!");
            ToKvid.ThrowIfEmpty("未传入粘贴报告的信息!");
            #endregion
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var connStandards = KiviiContext.GetOpenedDbConnection<Standard>();
            var connProducts = KiviiContext.GetOpenedDbConnection<Quotation>();
            #region 获取待粘贴报告相关信息
            //获取当前待粘贴报告信息
            var reportPaste = conn.SingleById<Report>(ToKvid);
            reportPaste.ThrowIfNull("未找到待粘贴报告!");
            //如果粘贴报告存在检验项目,禁止操作
            var queryReportItemsByReportPaste = conn.From<ReportItem>();
            queryReportItemsByReportPaste.Where(o => o.ReportKvid == ToKvid);
            var reportItemsByReportPaste = conn.Select(queryReportItemsByReportPaste);
            (reportItemsByReportPaste.Count != 0).ThrowIfTrue("待粘贴报告存在检验项目,禁止粘贴!");
            //获取当前待粘贴报告对应样品
            var querySamplesPaste = conn.From<Sample>();
            querySamplesPaste.Where(o => o.RootKvid == reportPaste.SampleKvid);
            var samplesPaste = conn.Select(querySamplesPaste);
            var samplePasteRoot = new Sample();
            var sampleKvidChildren = new List<Guid>();
            foreach (var samplePaste in samplesPaste)
            {
                if (samplePaste.Kvid == samplePaste.RootKvid) {
                    samplePasteRoot = samplePaste;
                    continue;
                } 
                sampleKvidChildren.Add(samplePaste.Kvid);
            }
            if (samplePasteRoot == null) throw new Exception("未匹配主样品信息!");
            //删除子样品数据
            if(sampleKvidChildren.Count>0) conn.DeleteByIds<Sample>(sampleKvidChildren);
            #endregion

            #region 获取拷贝报告相关信息
            //获取拷贝报告信息
            var reportCopy = conn.SingleById<Report>(FromKvid);
            reportCopy.ThrowIfNull("未找到拷贝报告!");
            //获取当前拷贝报告对应样品
            var querySamplesCopy = conn.From<Sample>();
            querySamplesCopy.Where(o => o.RootKvid == reportCopy.SampleKvid);
            var samplesCopy = conn.Select(querySamplesCopy);
            var sampleCopyRoot = samplesCopy.FirstOrDefault(o=>o.Kvid== reportCopy.SampleKvid);
            //获取当前拷贝报告对应的检验项目
            var queryReportItems = conn.From<ReportItem>();
            queryReportItems.Where(o => o.ReportKvid == FromKvid);
            var reportItems = conn.Select(queryReportItems);
            //获取当前拷贝报告对应的检验项目中涉及到的标准值信息
            var detectionKvids = reportItems.ConvertAll(o => o.DetectionKvid);
            var queryDetections = connStandards.From<Detection>();
            queryDetections.Where(o => Sql.In(o.Kvid, detectionKvids));
            var detections = connStandards.Select(queryDetections);
            //获取标准信息(涉及到标准信息变更)
            var standardKvids = detections.ConvertAll(o => o.StandardKvid).Distinct().ToList();
            var standards = connStandards.Select<Standard>(o => Sql.In(o.Kvid, standardKvids));
            //获取报告所涉及项目的标准值
            var queryDetectionValues = connStandards.From<DetectionValue>();
            queryDetectionValues.Where(o => Sql.In(o.DetectionKvid, detectionKvids));
            var detectionValues = connStandards.Select(queryDetectionValues);
            //获取报告涉及的合同、服务相关信息
            Quotation quotation = null;
            if (reportCopy.QuotationKvid != Guid.Empty) quotation = connProducts.SingleById<Quotation>(reportCopy.QuotationKvid);
            List<QuotationDetail> quotationDetails = new List<QuotationDetail>();
            if (quotation != null)
            {
                var queryQuotationDetails = connProducts.From<QuotationDetail>();
                queryQuotationDetails.Where(o => o.QuotationKvid == quotation.Kvid);
                quotationDetails = connProducts.Select(queryQuotationDetails);
            }
            //检验项目后改价格,按最新匹配 2024-03-25
            //goodsKvids:ReportItems的GoodsKvid集合+涉及Detections的GoodsKvid集合(存在标准检验项目后改价格的情况)
            var goodsKvidsByReportItem = reportItems.Select(o => o.GoodsKvid).Distinct().ToList();
            var goodsKvidsByDetections = detections.Select(o => o.GoodsKvid).Distinct().ToList();
            var goodsKvids = goodsKvidsByReportItem.Union(goodsKvidsByDetections).Distinct().ToList();
            var queryPrices = connProducts.From<Price>();
            queryPrices.Where(o => Sql.In(o.GoodsKvid, goodsKvids));
            var prices = connProducts.Select(queryPrices);
            #endregion

            var rtns = new RestfulQueryResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();
            IDbTransaction trans = null;
            trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);
            using (KiviiContext.Profiler($"粘贴检验项目：{reportCopy.ReportId}"))
            {
                try
                {
                    //2.粘贴所有项目
                    if (Type == PasteType.ReportItemsAll)
                    {
                        reportPaste.RemoveAllOnlyProperties();
                        if (reportCopy.Metadata != null && reportCopy.Metadata != null && reportCopy.Metadata.ContainsKey("testStandardsMethod"))
                        { 
                            if(reportPaste.Metadata==null)reportPaste.Metadata = new Dictionary<string, string>();
                            reportPaste.Metadata["testStandardsMethod"] = reportCopy.Metadata["testStandardsMethod"];
                            reportPaste.AddOnlyProperties(o => o.Metadata);
                        }
                        reportPaste.TestStandards = reportCopy.TestStandards;
                        reportPaste.AddOnlyProperties(o => o.TestStandards);
                        reportPaste.TestDetections = reportCopy.TestDetections;
                        reportPaste.AddOnlyProperties(o => o.TestDetections);
                        reportPaste.AmountTest = reportCopy.AmountTest;
                        reportPaste.AddOnlyProperties(o => o.AmountTest);
                        reportPaste.AmountTestPlan = reportCopy.AmountTestPlan;
                        reportPaste.AddOnlyProperties(o => o.AmountTestPlan);
                        conn.UpdateOnly(reportPaste);

                        //2.1 创建样品(可能存在子样品)
                        //获取样品新老Kvid集合
                        var sampleMatchKvids = new List<MatchKvids>();
                        sampleMatchKvids.Add(new MatchKvids
                        {
                            OriginalKvid = Guid.Empty,
                            MatchKvid = Guid.Empty
                        });
                        sampleMatchKvids.Add(new MatchKvids
                        {
                            OriginalKvid = reportCopy.SampleKvid,
                            MatchKvid = reportPaste.SampleKvid
                        });
                        //获取当前拷贝报告对应的子样品
                        //获取拷贝检验项目引用的样品集合
                        var sampleChildren = samplesCopy.Where(o => o.Kvid!=o.RootKvid).ToList();
                        Console.WriteLine($"拷贝子样品数量[{sampleChildren.Count}]");
                        foreach (var sampleChild in sampleChildren)
                        {
                            var sampleChildPaste = new Sample();
                            sampleChildPaste.PopulateWith(sampleChild);
                            sampleChildPaste.Kvid = Guid.NewGuid();
                            sampleChildPaste.RootKvid = samplePasteRoot.RootKvid;
                            sampleChildPaste.ParentKvid = samplePasteRoot.Kvid;
                            sampleChildPaste.SourceKvid = reportPaste.SampleSourceKvid;
                            sampleChildPaste.SourceBatchNumber = String.Empty;
                            sampleChildPaste.SerialNumber = sampleChild.SerialNumber.Replace(sampleCopyRoot.SerialNumber, samplePasteRoot.SerialNumber);
                            Console.WriteLine($"样品流水号-----拷贝子样品[{sampleChild.SerialNumber}]--拷贝报告流水[{sampleCopyRoot.SerialNumber}--粘贴报告流水[{samplePasteRoot.SerialNumber}]");
                            Console.WriteLine($"粘贴子样品流水号[{sampleChildPaste.SerialNumber}]");
                            conn.Insert(sampleChildPaste);
                            sampleMatchKvids.Add(new MatchKvids
                            {
                                OriginalKvid = sampleChild.Kvid,
                                MatchKvid = sampleChildPaste.Kvid
                            });
                        }
                        //2.2 创建检验项目
                        //获取检验项目新老Kvid集合
                        var reportItemsKvids = reportItems.Select(o => o.Kvid).ToList();
                        var reportItemMatchKvids = new List<MatchKvids>();
                        reportItemMatchKvids.Add(new MatchKvids
                        {
                            OriginalKvid = Guid.Empty,
                            MatchKvid = Guid.Empty
                        });
                        foreach (var reportItemKvid in reportItemsKvids)
                        {
                            reportItemMatchKvids.Add(new MatchKvids
                            {
                                OriginalKvid = reportItemKvid,
                                MatchKvid = Guid.NewGuid()
                            });
                        }
                        foreach (var reportItem in reportItems)
                        {
                            var detection = detections.Where(o => o.Kvid == reportItem.DetectionKvid).FirstOrDefault();
                            var pasteReportItem = reportItem.GetPasteReportItemUpgrade(reportPaste, detection, detectionValues, standards, IsCopyDeteValueCustomerRequired);
                            Console.WriteLine("OK1");
                            pasteReportItem.RootKvid = reportItem.RootKvid.GetMatchKvid(reportItemMatchKvids);
                            Console.WriteLine("OK1");
                            pasteReportItem.ParentKvid = reportItem.ParentKvid.GetMatchKvid(reportItemMatchKvids);
                            pasteReportItem.Kvid = reportItem.Kvid.GetMatchKvid(reportItemMatchKvids);
                            pasteReportItem.ReportKvid = reportPaste.Kvid;
                            pasteReportItem.SampleKvid = reportItem.SampleKvid.GetMatchKvid(sampleMatchKvids);
                            pasteReportItem.SampleRootKvid = reportItem.SampleRootKvid.GetMatchKvid(sampleMatchKvids);
                            pasteReportItem.CalculateAmount(quotation, quotationDetails, prices);
                            conn.Insert(pasteReportItem);
                            pasteReportItem.RemoveAllOnlyProperties();
                            rtns.Results.Add(pasteReportItem);
                            //创建标准值日志
                            pasteReportItem.SetDetectionValueQualifiedLog("create", conn, null, null, "ReportItemPaste");
                            //创建客户要求标准值日志
                            if (pasteReportItem.Metadata != null && pasteReportItem.Metadata.ContainsKey("DetectionValueCustomer") && !pasteReportItem.Metadata["DetectionValueCustomer"].IsNullOrEmpty()) pasteReportItem.SetDetectionValueQualifiedCustomerRequiredLog("create", null, conn);
                        }
                    }
                    //3.粘贴原项目
                    if (Type == PasteType.ReportItemsOriginal)
                    {
                        //找到主色号下的所有项
                        var reportItemRoot = reportItems.Where(o => o.SampleKvid == o.SampleRootKvid && o.Type == ReportItemType.SampleSelf).FirstOrDefault();
                        //获取需要拷贝的所有项目
                        var reportItemsOriginal = new List<ReportItem>();
                        var isCopyAllReportItems = false;
                        if (reportItemRoot == null)
                        {
                            reportItemsOriginal = reportItems;
                            isCopyAllReportItems = true;
                        }
                        else
                        {
                            //获取主色号下的所有项目
                            reportItemsOriginal = reportItems.Where(o => o.RootKvid == reportItemRoot.Kvid && o.ParentKvid != Guid.Empty).ToList();
                        }
                        //获取检验项目新老Kvid集合
                        var reportItemOriginalKvids = reportItemsOriginal.Select(o => o.Kvid).ToList();
                        var reportItemOriginalMatchKvids = new List<MatchKvids>();
                        reportItemOriginalMatchKvids.Add(new MatchKvids
                        {
                            OriginalKvid = Guid.Empty,
                            MatchKvid = Guid.Empty
                        });
                        foreach (var reportItemKvid in reportItemOriginalKvids)
                        {
                            reportItemOriginalMatchKvids.Add(new MatchKvids
                            {
                                OriginalKvid = reportItemKvid,
                                MatchKvid = Guid.NewGuid()
                            });
                        }
                        if (IsContainParts)
                        {
                            //包含部位
                            //创建检验项目
                            foreach (var reportItemOriginal in reportItemsOriginal)
                            {
                                var detection = detections.Where(o => o.Kvid == reportItemOriginal.DetectionKvid).FirstOrDefault();
                                var pasteReportItem = reportItemOriginal.GetPasteReportItemUpgrade(reportPaste, detection, detectionValues, standards, IsCopyDeteValueCustomerRequired);
                                //判断主项/子项
                                var newRootKvid = new Guid();
                                var newParentKvid = new Guid();
                                if (isCopyAllReportItems)
                                {
                                    newRootKvid = reportItemOriginal.RootKvid;
                                    newParentKvid = reportItemOriginal.ParentKvid;
                                }
                                else
                                {
                                    if (reportItemOriginal.ParentKvid == reportItemOriginal.RootKvid)
                                    {
                                        //主项
                                        newRootKvid = reportItemOriginal.Kvid;
                                        newParentKvid = Guid.Empty;
                                    }
                                    else
                                    {
                                        //子项,子项的RootKvid需要特殊处理
                                        newRootKvid = reportItemOriginal.GetRootKvid(reportItemsOriginal);
                                        newParentKvid = reportItemOriginal.ParentKvid;
                                    }
                                }
                                pasteReportItem.RootKvid = newRootKvid.GetMatchKvid(reportItemOriginalMatchKvids);
                                pasteReportItem.ParentKvid = newParentKvid.GetMatchKvid(reportItemOriginalMatchKvids);
                                pasteReportItem.Kvid = reportItemOriginal.Kvid.GetMatchKvid(reportItemOriginalMatchKvids);
                                pasteReportItem.ReportKvid = reportPaste.Kvid;
                                pasteReportItem.SampleKvid = samplePasteRoot.Kvid;
                                pasteReportItem.SampleRootKvid = samplePasteRoot.RootKvid;
                                pasteReportItem.CalculateAmount(quotation, quotationDetails, prices);
                                conn.Insert(pasteReportItem);
                                pasteReportItem.RemoveAllOnlyProperties();
                                rtns.Results.Add(pasteReportItem);
                                //创建标准值日志
                                pasteReportItem.SetDetectionValueQualifiedLog("create", conn, null, null, "ReportItemPaste");
                                //创建客户要求标准值日志
                                if (pasteReportItem.Metadata != null && pasteReportItem.Metadata.ContainsKey("DetectionValueCustomer") && !pasteReportItem.Metadata["DetectionValueCustomer"].IsNullOrEmpty()) pasteReportItem.SetDetectionValueQualifiedCustomerRequiredLog("create", null, conn);
                            }

                        }
                        else
                        {
                            //不包含部位,提取所有DetectionKvid不为空的数据
                            //只保留主项(去重)->处理对应子项->处理剩下的Type为Test的主项
                            reportItemsOriginal = reportItemsOriginal.Where(o => o.DetectionKvid != Guid.Empty).ToList();
                            var reportItemsOriginalTestFolder = reportItemsOriginal.Where(o => o.Type == ReportItemType.TestFolder).ToList();
                            reportItemsOriginal = reportItemsOriginal.Except(reportItemsOriginalTestFolder).ToList();
                            //记录处理过的主项值
                            var detectionKvidsExist = new List<Guid>();
                            foreach (var reportItemOriginalTestFolder in reportItemsOriginalTestFolder)
                            {
                                //如遇相同主项直接跳过
                                if (detectionKvidsExist.Contains(reportItemOriginalTestFolder.DetectionKvid))
                                {
                                    reportItemsOriginal.Remove(reportItemOriginalTestFolder);
                                    var reportItemsTestFolderChildren = reportItemsOriginal.Where(o => o.ParentKvid == reportItemOriginalTestFolder.Kvid).ToList();
                                    reportItemsOriginal = reportItemsOriginal.Except(reportItemsTestFolderChildren).ToList();
                                    continue;
                                }
                                detectionKvidsExist.Add(reportItemOriginalTestFolder.DetectionKvid);
                                //创建主项
                                var detection = detections.Where(o => o.Kvid == reportItemOriginalTestFolder.DetectionKvid).FirstOrDefault();
                                var pasteReportItemParent = reportItemOriginalTestFolder.GetPasteReportItemUpgrade(reportPaste, detection, detectionValues, standards, IsCopyDeteValueCustomerRequired);
                                pasteReportItemParent.RootKvid = reportItemOriginalTestFolder.Kvid.GetMatchKvid(reportItemOriginalMatchKvids);
                                pasteReportItemParent.ParentKvid = Guid.Empty;
                                pasteReportItemParent.Kvid = reportItemOriginalTestFolder.Kvid.GetMatchKvid(reportItemOriginalMatchKvids);
                                pasteReportItemParent.ReportKvid = reportPaste.Kvid;
                                pasteReportItemParent.SampleKvid = samplePasteRoot.Kvid;
                                pasteReportItemParent.SampleRootKvid = samplePasteRoot.RootKvid;
                                pasteReportItemParent.CalculateAmount(quotation, quotationDetails, prices);
                                conn.Insert(pasteReportItemParent);
                                pasteReportItemParent.RemoveAllOnlyProperties();
                                rtns.Results.Add(pasteReportItemParent);
                                reportItemsOriginal.Remove(reportItemOriginalTestFolder);
                                //创建标准值日志
                                pasteReportItemParent.SetDetectionValueQualifiedLog("create", conn, null, null, "ReportItemPaste");
                                //创建客户要求标准值日志
                                if (pasteReportItemParent.Metadata != null && pasteReportItemParent.Metadata.ContainsKey("DetectionValueCustomer") && !pasteReportItemParent.Metadata["DetectionValueCustomer"].IsNullOrEmpty()) pasteReportItemParent.SetDetectionValueQualifiedCustomerRequiredLog("create", null, conn);
                                //创建子项
                                var reportItemsOriginalChildren = reportItemsOriginal.Where(o => o.ParentKvid == reportItemOriginalTestFolder.Kvid).ToList();
                                foreach (var reportItemChild in reportItemsOriginalChildren)
                                {
                                    var detectionChild = detections.Where(o => o.Kvid == reportItemChild.DetectionKvid).FirstOrDefault();
                                    var pasteReportItemChild = reportItemChild.GetPasteReportItemUpgrade(reportPaste, detectionChild, detectionValues, standards, IsCopyDeteValueCustomerRequired);
                                    pasteReportItemChild.RootKvid = pasteReportItemParent.Kvid;
                                    pasteReportItemChild.ParentKvid = pasteReportItemParent.Kvid;
                                    pasteReportItemChild.Kvid = reportItemChild.Kvid.GetMatchKvid(reportItemOriginalMatchKvids);
                                    pasteReportItemChild.ReportKvid = reportPaste.Kvid;
                                    pasteReportItemChild.SampleKvid = samplePasteRoot.Kvid;
                                    pasteReportItemChild.SampleRootKvid = samplePasteRoot.RootKvid;
                                    pasteReportItemChild.CalculateAmount(quotation, quotationDetails, prices);
                                    conn.Insert(pasteReportItemChild);
                                    pasteReportItemChild.RemoveAllOnlyProperties();
                                    rtns.Results.Add(pasteReportItemChild);
                                    reportItemsOriginal.Remove(reportItemChild);
                                    //创建标准值日志
                                    pasteReportItemChild.SetDetectionValueQualifiedLog("create", conn, null, null, "ReportItemPaste");
                                    //创建客户要求标准值日志
                                    if (pasteReportItemChild.Metadata != null && pasteReportItemChild.Metadata.ContainsKey("DetectionValueCustomer") && !pasteReportItemChild.Metadata["DetectionValueCustomer"].IsNullOrEmpty()) pasteReportItemChild.SetDetectionValueQualifiedCustomerRequiredLog("create", null, conn);
                                }

                            }
                            //处理剩余项目:Type为Test的主项
                            foreach (var reportItem in reportItemsOriginal)
                            {
                                //如遇相同主项直接跳过
                                if (detectionKvidsExist.Contains(reportItem.DetectionKvid)) continue;
                                detectionKvidsExist.Add(reportItem.DetectionKvid);
                                var detection = detections.Where(o => o.Kvid == reportItem.DetectionKvid).FirstOrDefault();
                                var pasteReportItem = reportItem.GetPasteReportItemUpgrade(reportPaste, detection, detectionValues, standards, IsCopyDeteValueCustomerRequired);
                                pasteReportItem.RootKvid = reportItem.Kvid.GetMatchKvid(reportItemOriginalMatchKvids);
                                pasteReportItem.ParentKvid = Guid.Empty;
                                pasteReportItem.Kvid = reportItem.Kvid.GetMatchKvid(reportItemOriginalMatchKvids);
                                pasteReportItem.ReportKvid = reportPaste.Kvid;
                                pasteReportItem.SampleKvid = samplePasteRoot.Kvid;
                                pasteReportItem.SampleRootKvid = samplePasteRoot.RootKvid;
                                pasteReportItem.CalculateAmount(quotation, quotationDetails, prices);
                                conn.Insert(pasteReportItem);
                                pasteReportItem.RemoveAllOnlyProperties();
                                rtns.Results.Add(pasteReportItem);
                                //创建标准值日志
                                pasteReportItem.SetDetectionValueQualifiedLog("create", conn,null,null, "ReportItemPaste");
                                //创建客户要求标准值日志
                                if (pasteReportItem.Metadata != null && pasteReportItem.Metadata.ContainsKey("DetectionValueCustomer") && !pasteReportItem.Metadata["DetectionValueCustomer"].IsNullOrEmpty()) pasteReportItem.SetDetectionValueQualifiedCustomerRequiredLog("create", null, conn);
                            }
                        }
                    }
                    rtns.Total = rtns.Results.Count;
                    trans?.Commit();
                    //事务提交后，同步报告金额
                    reportPaste.CalculateReportAmount(conn);
                    conn.UpdateOnly(reportPaste);
                    return rtns;
                }
                catch (Exception ex)
                {
                    trans?.Rollback();
                    throw ex;
                }
            }
        }
    }
    #endregion

    #region lims下样选择检验项目同步价格、同步检验组
    /// <summary>
    /// 通过浏览标准项目生成检验项目
    /// 同步设置生成项目的价格、检验组信息
    /// 2020.11.17更新，新增标准排序号
    /// </summary>
    //代码初审 张梦圆 20210907
    [Api(Description = "项目选择")]
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class ReportItemDetection : RestfulExecution<ReportItem>
    {
        public Guid ReportKvid { get; set; }
        public Guid ReportItemKvid { get; set; }//是否挂载在色号/部位下
        public List<Detection> Detections { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            #region 数据验证
            ReportKvid.ThrowIfEmpty("未传入ReportKvid!");
            Detections.ThrowIfNullOrEmpty("未传入检测项目或检测项目为空!");
            #endregion
            var connLims = KiviiContext.GetOpenedDbConnection<Report>();
            //获取当前报告信息
            var report = connLims.SingleById<Report>(ReportKvid);
            report.ThrowIfNull("未找到对应报告!");
            if (report.StatusType > ReportStatusType.CommissionAccept) throw new Exception("当前报告不在委托受理，无法添加项目!");
            //获取当前报告关联的样品集合
            var querySamples = connLims.From<Sample>();
            querySamples.Where(o => o.RootKvid == report.SampleKvid);
            var samples = connLims.Select(querySamples);
            (samples.IsNullOrEmpty()).ThrowIfTrue("未找到对应样品!");
            //获取当前报告检验项目集合
            var queryReportItems = connLims.From<ReportItem>();
            queryReportItems.Where(o => o.ReportKvid == ReportKvid);
            var reportItems = connLims.Select(queryReportItems);
            //如果ReportItemKvid参数为空，应验证是否存在色号，因为若分色号了，所有项目均需要挂在色号下
            if (ReportItemKvid == null || ReportItemKvid == Guid.Empty)
            {
                reportItems.Exists(o => o.Type == ReportItemType.SampleSelf).ThrowIfTrue("请选择您要存放所选检验项目的色号！");
            }
            //获取当前选中的ReportItem--如果选中项类型为SampleSelf/SamplePart/TestFolder/Test(必须为大项)/Folder,则在其目录下添加项目,否则当未选中项目处理
            //选中项目为Folder类型的暂不考虑(暂未使用,使用时根据具体逻辑做相应处理)
            //如果有选中的项目,获取其目录下项目集合
            ReportItem reportItemSelected = null;
            if (ReportItemKvid != null && ReportItemKvid != Guid.Empty)
            {
                reportItemSelected = connLims.SingleById<ReportItem>(ReportItemKvid);
                reportItemSelected.ThrowIfNull("未找到选中的检验项目!");
                if (reportItemSelected.Type == ReportItemType.SampleSelf)
                {
                    //若选中的项目为色号,获取色号下所有检验项目(包括子项)
                    reportItems = reportItems.Where(o => o.RootKvid == ReportItemKvid && o.Kvid != ReportItemKvid).ToList();
                }
                if (reportItemSelected.Type == ReportItemType.SamplePart)
                {
                    //若选中的项目为部位,获取部位下所有检验项目
                    var reportItemsParent = reportItems.Where(o => o.ParentKvid == ReportItemKvid).ToList();
                    var reportItemKvidsParent = reportItemsParent.ConvertAll(o => o.Kvid);
                    var reportItemsChildren = reportItems.Where(o => Sql.In(o.ParentKvid, reportItemKvidsParent)).ToList();
                    reportItems = reportItemsParent.Union(reportItemsChildren).ToList();
                }
                if (reportItemSelected.Type == ReportItemType.TestFolder)
                {
                    //若选中的项目为大项,获取该项目下的所有子项
                    reportItems = reportItems.Where(o => o.ParentKvid == ReportItemKvid).ToList();
                }
                if (reportItemSelected.Type == ReportItemType.Test)
                {
                    //若选中的项目为大项,则该项目下不该有任何子项
                    if (reportItemSelected.ParentKvid == Guid.Empty)
                    {
                        reportItems.Clear();
                    }
                    else
                    {
                        var reportItemParent = reportItems.FirstOrDefault(o => o.Kvid == reportItemSelected.ParentKvid);
                        reportItemParent.ThrowIfNull("未匹配到选中项目的父节点!无法添加项目!");
                        if (reportItemParent.Type != ReportItemType.TestFolder)
                        {
                            reportItems.Clear();
                        }
                        else
                        {
                            reportItemSelected = null;
                        }
                    }
                }
            }
            //过滤当前已有项目
            var detectionKvidsExists = reportItems.Select(o => o.DetectionKvid).Distinct().ToList();
            Detections = Detections.Where(o => !Sql.In(o.Kvid, detectionKvidsExists)).ToList();
            Detections.IsNullOrEmpty().ThrowIfTrue("所选均为重复项,已忽略！");
            //如果选中项类型为TestFolder/Test,还需要验证当前选中的检验项目
            if (reportItemSelected != null && (reportItemSelected.Type == ReportItemType.TestFolder || (reportItemSelected.Type == ReportItemType.Test && reportItems.IsNullOrEmpty())))
            {
                if (Detections.Exists(o => o.ParentKvid == Guid.Empty || o.ParentKvid != reportItemSelected.DetectionKvid)) throw new Exception("选择的检验项目不在可创建范围,请重新选择!");
            }
            //获取标准信息和标准值信息
            var connStandards = KiviiContext.GetOpenedDbConnection<Detection>();
            var standardKvids = Detections.ConvertAll(o => o.StandardKvid).Distinct().ToList();
            var standards = connStandards.Select<Standard>(o => Sql.In(o.Kvid, standardKvids));
            standards.IsNullOrEmpty().ThrowIfTrue("未匹配到对应标准!");
            var detectionsKvids = Detections.ConvertAll(o => o.Kvid);
            var detectionValues = connStandards.Select<DetectionValue>(o => Sql.In(o.DetectionKvid, detectionsKvids));
            //获取报价信息
            var goodsKvids = Detections.ConvertAll(o => o.GoodsKvid);
            //如果为引用项目,追溯到原生项,提取原生项的GoodsKvids集合
            var referenceDetectionKvids = Detections.ConvertAll(o => o.ReferenceDetectionKvid);
            var referenceDetections = connStandards.Select<Detection>(o => Sql.In(o.Kvid, referenceDetectionKvids));
            if (!referenceDetections.IsNullOrEmpty())
            {
                var goodsKvidsByReferenceDetections = referenceDetections.ConvertAll(o => o.GoodsKvid);
                goodsKvids = goodsKvids.Union(goodsKvidsByReferenceDetections).Distinct().ToList();
            }
            var connProduct = KiviiContext.GetOpenedDbConnection<Quotation>();
            List<Price> prices = null;
            Quotation quotation = null;//客户报价单
            List<QuotationDetail> quotationDetails = null;
            prices = connProduct.Select<Price>(o => Sql.In(o.GoodsKvid, goodsKvids));
            if (report.QuotationKvid != Guid.Empty)
            {
                quotation = connProduct.SingleById<Quotation>(report.QuotationKvid);
                quotationDetails = connProduct.Select<QuotationDetail>(o => o.QuotationKvid == report.QuotationKvid && Sql.In(o.GoodsKvid, goodsKvids));
            }

            //获取已存在项目中最大的SortID
            var sortIdMax = 0;
            if (reportItemSelected == null)
            {
                var reportItemsParent = reportItems.Where(o => o.ParentKvid == Guid.Empty).ToList();
                if (reportItemsParent.Count != 0) sortIdMax = reportItemsParent.OrderByDescending(o => o.SortId).FirstOrDefault().SortId;
                //如果SortIdMAX<0,说明当前已有项目都是强标,非强标排序号从0开始
                if (sortIdMax < 0) sortIdMax = 0;
            }
            else
            {
                if (reportItemSelected.Type == ReportItemType.SampleSelf || reportItemSelected.Type == ReportItemType.SamplePart)
                {
                    var reportItemsParent = reportItems.Where(o => o.ParentKvid == reportItemSelected.Kvid).ToList();
                    if (reportItemsParent.Count != 0) sortIdMax = reportItemsParent.OrderByDescending(o => o.SortId).FirstOrDefault().SortId;
                }
                if (reportItemSelected.Type == ReportItemType.TestFolder)
                {
                    sortIdMax = reportItemSelected.SortId;
                }
                if (reportItemSelected.Type == ReportItemType.Test && reportItems.IsNullOrEmpty())
                {
                    //若是Type类型添加子项,自身的Type要改为TypeFolder
                    sortIdMax = reportItemSelected.SortId;
                    reportItemSelected.Type = ReportItemType.TestFolder;
                    reportItemSelected.AddOnlyProperties(o => o.Type);
                }
                //如果SortIdMAX<0,说明当前已有项目都是强标,非强标排序号从0开始
                if (sortIdMax < 0) sortIdMax = 0;
            }
            //SortIdEx需要单独处理,目前是原SortId*100,为生成部位时排序号的特殊处理做准备
            var rtns = new RestfulQueryResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();
            var reportItemsOperate = new List<ReportItem>();
            var groups = Detections.GroupBy(o => o.StandardKvid);
            foreach (var group in groups)
            {
                foreach (var detection in group)
                {
                    var standard = standards.FirstOrDefault(o => o.Kvid == detection.StandardKvid);
                    standard.ThrowIfNull("未匹配到对应标准!");
                    //如果选中项类型为TestFolder和Test(大项),先处理其子项
                    //其余子项稍后处理
                    if (detection.ParentKvid != Guid.Empty)
                    {
                        if (reportItemSelected != null && (reportItemSelected.Type == ReportItemType.TestFolder || reportItemSelected.Type == ReportItemType.Test))
                        {
                            var reportItemChild = new ReportItem();
                            reportItemChild.ParentKvid = reportItemSelected.Kvid;
                            reportItemChild.RootKvid = reportItemSelected.RootKvid;
                            reportItemChild.ReportKvid = report.Kvid;
                            reportItemChild.SampleKvid = reportItemSelected.SampleKvid;
                            reportItemChild.SampleRootKvid = reportItemSelected.SampleRootKvid;
                            reportItemChild.Type = ReportItemType.Test;
                            reportItemChild.StandardKvid = standard.Kvid;
                            reportItemChild.DetectionKvid = detection.Kvid;
                            reportItemChild.DetectionTitle = detection.Title;
                            reportItemChild.DetectionReferenceKvid = detection.ReferenceDetectionKvid;
                            reportItemChild.WorkGroupName = reportItemSelected.WorkGroupName;
                            reportItemChild.WorkGroupKvid = reportItemSelected.WorkGroupKvid;
                            reportItemChild.Title = detection.Title;
                            reportItemChild.Currency = CurrencyUnit.CNY;
                            reportItemChild.SortIdEx = detection.SortId * 100;
                            reportItemChild.SortId = reportItemSelected.SortId;
                            reportItemChild.DetectionUnit = detection.Unit;
                            reportItemChild.DetectionValueQualified = detection.GetQualifiedValue(report.SampleGrade, report.SampleGradeSafety, detectionValues);
                            reportItemChild.Metadata = new Dictionary<string, string>();
                            //2022.12.14 将标准的项目说明赋值给子项Metadata
                            reportItemChild.Metadata.Add("DetectionDescription", detection.Description);
                            reportItemChild.Metadata.Add("IsMultiReference", detection.IsMultiReference.ToString());
                            reportItemChild.Status = (int)ReportItemStatusType.BeforeTest;
                            reportItemsOperate.Add(reportItemChild);
                            reportItems.Add(reportItemChild);
                        }
                        continue;
                    }
                    var detectionsChildren = Detections.Where(o => o.ParentKvid == detection.Kvid).ToList();
                    //创建大项
                    var reportItemParent = new ReportItem();
                    reportItemParent.Metadata = new Dictionary<string, string>();
                    reportItemParent.Kvid = Guid.NewGuid();
                    reportItemParent.ParentKvid = reportItemSelected != null ? reportItemSelected.Kvid : Guid.Empty;
                    reportItemParent.RootKvid = reportItemSelected != null ? reportItemSelected.RootKvid : reportItemParent.Kvid;
                    reportItemParent.ReportKvid = ReportKvid;
                    reportItemParent.SampleKvid = reportItemSelected != null ? reportItemSelected.SampleKvid : report.SampleKvid;
                    reportItemParent.SampleRootKvid = report.SampleKvid;
                    reportItemParent.Type = detectionsChildren.IsNullOrEmpty() ? ReportItemType.Test : ReportItemType.TestFolder;
                    reportItemParent.StandardKvid = standard.Kvid;
                    reportItemParent.StandardCode = standard.Code;
                    reportItemParent.StandardTitle = standard.Title;
                    reportItemParent.DetectionKvid = detection.Kvid;
                    //2020.11.13讨论将detection的CodeActual赋值给ReportItem的DetectionCode
                    //reportItemParent.DetectionCodeActual = (detection.CodeActual.Split(';').Length > 1) ? detection.CodeActual.Split(';')[0] : detection.CodeActual;
                    //2025-01-16 因一个项目可能引用多个测试方法，默认第一个测试方法，会大概率导致录入员未注意到引用多个，导致差错，故改成默认全部
                    reportItemParent.DetectionCodeActual = detection.CodeActual;
                    reportItemParent.Metadata.Add("DetectionCodeActual", detection.CodeActual);
                    if (standard.Metadata!=null && standard.Metadata.ContainsKey("TitleEx"))
                    {
                        reportItemParent.Metadata.Add("StandardTitleEx", standard.Metadata["TitleEx"]);
                    }
                    if (!detection.MethodDescription.IsNullOrEmpty()) reportItemParent.DetectionCodeActual = reportItemParent.DetectionCodeActual + " " + detection.MethodDescription;
                    reportItemParent.DetectionTitle = detection.Title;
                    reportItemParent.DetectionReferenceKvid = detection.ReferenceDetectionKvid;
                    reportItemParent.Title = detection.Title;
                    reportItemParent.SortIdEx = detection.SortId * 100;
                    //设置SortId
                    //针对强标(31701/18401/5296.4)需要特殊处理(原代码处理:18401和31701平级,SortId设置为-1;5296.4设置为-2)
                    //如果存在强标,获取已存在强标的最小排序号
                    if (standard.Code.Contains("18401"))
                    {
                        if (reportItems.Exists(o => o.StandardKvid == detection.StandardKvid))
                        {
                            reportItemParent.SortId = reportItems.FirstOrDefault(o => o.StandardKvid == detection.StandardKvid).SortId;
                        }
                        else
                        {
                            //先将StandardCode为空的数据过滤掉再判断,否则会报错
                            var reportItemSpecial = reportItems.OrderByDescending(o => o.SortId).Where(o => o.StandardCode != null).ToList().FirstOrDefault(o => o.StandardCode.Contains("18401"));
                            var sortId = reportItemSpecial == null ? -100 : reportItemSpecial.SortId + 1;
                            reportItemParent.SortId = sortId;
                        }
                    }
                    else if (standard.Code.Contains("31701"))
                    {
                        if (reportItems.Exists(o => o.StandardKvid == detection.StandardKvid))
                        {
                            reportItemParent.SortId = reportItems.FirstOrDefault(o => o.StandardKvid == detection.StandardKvid).SortId;
                        }
                        else
                        {
                            var reportItemSpecial = reportItems.OrderByDescending(o => o.SortId).Where(o => o.StandardCode != null).ToList().FirstOrDefault(o => o.StandardCode.Contains("31701"));
                            var sortId = reportItemSpecial == null ? -200 : reportItemSpecial.SortId + 1;
                            reportItemParent.SortId = sortId;
                        }
                    }
                    else if (standard.Code.Contains("5296.4"))
                    {
                        if (reportItems.Exists(o => o.StandardKvid == detection.StandardKvid))
                        {
                            reportItemParent.SortId = reportItems.FirstOrDefault(o => o.StandardKvid == detection.StandardKvid).SortId;
                        }
                        else
                        {

                            var reportItemSpecial = reportItems.OrderByDescending(o => o.SortId).Where(o => o.StandardCode != null).ToList().FirstOrDefault(o => o.StandardCode.Contains("5296.4"));
                            var sortId = reportItemSpecial == null ? -300 : reportItemSpecial.SortId + 1;
                            reportItemParent.SortId = sortId;
                        }
                    }
                    else
                    {
                        if (reportItems.Exists(o => o.StandardKvid == detection.StandardKvid))
                        {
                            reportItemParent.SortId = reportItems.FirstOrDefault(o => o.StandardKvid == detection.StandardKvid).SortId;
                        }
                        else
                        {
                            sortIdMax++;
                            reportItemParent.SortId = sortIdMax;
                        }
                    }
                    reportItemParent.Currency = CurrencyUnit.CNY;
                    reportItemParent.DetectionUnit = detection.Unit;
                    reportItemParent.DetectionValueQualified = detection.GetQualifiedValue(report.SampleGrade, report.SampleGradeSafety, detectionValues);
                    //2021.1.15讨论将标准的方法说明和项目说明赋值给Metadata
                    reportItemParent.Metadata.Add("DetectionDescription", detection.Description);
                    reportItemParent.Metadata.Add("DetectionMethodDescription", detection.MethodDescription);
                    reportItemParent.Metadata.Add("IsMultiReference", detection.IsMultiReference.ToString());
                    reportItemParent.Status = (int)ReportItemStatusType.BeforeTest;
                    //设置价格信息
                    if (detection.GoodsKvid != Guid.Empty)
                    {
                        reportItemParent.GoodsKvid = detection.GoodsKvid;
                    }
                    else
                    {
                        reportItemParent.GoodsKvid = Guid.Empty;
                        if (detection.ReferenceDetectionKvid != Guid.Empty && !referenceDetections.IsNullOrEmpty())
                        {
                            var refenceDetection = referenceDetections.FirstOrDefault(o => o.Kvid == detection.ReferenceDetectionKvid);
                            if (refenceDetection != null) reportItemParent.GoodsKvid = refenceDetection.GoodsKvid;
                        }
                    }
                    if (detection.GoodsKvid != Guid.Empty && !prices.IsNullOrEmpty())
                    {
                        var currentPrices = prices.Where(o => o.GoodsKvid == detection.GoodsKvid).OrderByDescending(o => o.IsDefault).ToList();
                        if (!currentPrices.IsNullOrEmpty())
                        {
                            Price price = null;
                            if (currentPrices.Count == 1)
                            {
                                price = currentPrices[0];
                                reportItemParent.PackageKvid = price.PackageKvid;
                                reportItemParent.PackageUnit = price.PackageUnit;
                                reportItemParent.AmountPlan = price.Amount;
                            }
                            if (currentPrices.Count > 1)
                            {
                                price = currentPrices.FirstOrDefault(o => o.IsDefault);
                                if (price == null) price = currentPrices[0];
                                reportItemParent.PackageKvid = price.PackageKvid;
                                var packageUnitArray = new List<string>();
                                foreach (var currentPrice in currentPrices)
                                {
                                    packageUnitArray.Add(currentPrice.PackageUnit);
                                }
                                reportItemParent.PackageUnit = string.Join(",", packageUnitArray);
                                reportItemParent.AmountPlan = price.Amount;
                            }
                            QuotationDetail quotationDetail = null;
                            if (quotationDetails == null || price == null)
                            {
                                quotationDetail = null;
                            }
                            else
                            {
                                quotationDetail = quotationDetails.FirstOrDefault(o => o.PriceKvid == price.Kvid);
                            }
                            //如果没有客户报价详细，直接用折扣
                            if (quotationDetail == null)
                            {
                                if (quotation != null)
                                {
                                    reportItemParent.Amount = (decimal)(price.Amount * quotation.DiscountRate);
                                }
                                else
                                {
                                    reportItemParent.Amount = price.Amount;
                                }
                            }
                            else
                            {
                                reportItemParent.Amount = quotationDetail.Amount;
                            }
                        }

                    }
                    //设置检验组信息
                    if (detection.TestGroupFullName.IsNullOrEmpty())
                    {
                        reportItemParent.WorkGroupName = "暂未关联检验组";
                        reportItemParent.WorkGroupKvid = Guid.Empty;
                    }
                    else {
                        reportItemParent.WorkGroupName = detection.TestGroupFullName;
                        reportItemParent.WorkGroupKvid = detection.TestGroupKvid;
                    }
                    //var queryTaskInfo = connStandards.From<EntityRelation<Detection>>();
                    //queryTaskInfo.Where(o => o.EntityKvid == detection.Kvid && o.RelationType == "Kivii.Organizations.Entities.Organization");
                    //var taskInfo = connStandards.Select(queryTaskInfo).FirstOrDefault();
                    //if (taskInfo == null)
                    //{
                    //    reportItemParent.WorkGroupName = "暂未关联检验组";
                    //    reportItemParent.WorkGroupKvid = Guid.Empty;
                    //}
                    //else
                    //{
                    //    reportItemParent.WorkGroupName = taskInfo.InternalCode;
                    //    reportItemParent.WorkGroupKvid = taskInfo.RelationKvid;
                    //}
                    reportItemsOperate.Add(reportItemParent);
                    reportItems.Add(reportItemParent);
                    //创建子项
                    foreach (var detectionChild in detectionsChildren)
                    {
                        var reportItemChild = new ReportItem();
                        reportItemChild.RootKvid = reportItemParent.RootKvid;
                        reportItemChild.ParentKvid = reportItemParent.Kvid;
                        reportItemChild.ReportKvid = reportItemParent.ReportKvid;
                        reportItemChild.SampleKvid = reportItemParent.SampleKvid;
                        reportItemChild.SampleRootKvid = reportItemParent.SampleRootKvid;
                        reportItemChild.Type = ReportItemType.Test;
                        reportItemChild.StandardKvid = detection.StandardKvid;
                        reportItemChild.DetectionKvid = detectionChild.Kvid;
                        reportItemChild.DetectionReferenceKvid = detectionChild.ReferenceDetectionKvid;
                        reportItemChild.WorkGroupName = reportItemParent.WorkGroupName;
                        reportItemChild.WorkGroupKvid = reportItemParent.WorkGroupKvid;
                        reportItemChild.Title = detectionChild.Title;
                        //reportItemChild.StandardCode = detectionChild.Code;
                        reportItemChild.Currency = CurrencyUnit.CNY;
                        reportItemChild.SortIdEx = detectionChild.SortId * 100;
                        reportItemChild.SortId = reportItemParent.SortId;
                        reportItemChild.DetectionUnit = detectionChild.Unit;
                        reportItemChild.DetectionValueQualified = detectionChild.GetQualifiedValue(report.SampleGrade, report.SampleGradeSafety, detectionValues);
                        reportItemChild.Metadata = new Dictionary<string, string>();
                        //2022.12.14 将标准的项目说明赋值给子项Metadata
                        reportItemChild.Metadata.Add("DetectionDescription", detectionChild.Description);
                        reportItemChild.Metadata.Add("IsMultiReference", detectionChild.IsMultiReference.ToString());
                        reportItemChild.Status = (int)ReportItemStatusType.BeforeTest;
                        reportItemsOperate.Add(reportItemChild);
                        reportItems.Add(reportItemChild);
                    }
                }
            }
            IDbTransaction trans = null;//define a null tag
             trans = connLims.OpenTransaction();
            try
            {
                //是否更新之前选中的检验项目类型
                //这里就不写进返回值里了
                if (reportItemSelected != null && reportItemSelected.OnlyProperties.Count > 0) connLims.UpdateOnly(reportItemSelected);
                foreach (var reportItemOperate in reportItemsOperate)
                {
                    connLims.Insert(reportItemOperate);
                    reportItemOperate.RemoveAllOnlyProperties();
                    rtns.Results.Add(reportItemOperate);
                    //创建标准值日志
                    reportItemOperate.SetDetectionValueQualifiedLog("create",connLims,null,null, "ReportItemDetection");
                }
                trans?.Commit();
                rtns.Total = rtns.Results.Count;
                return rtns;
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }
        }
    }

    #endregion

    #region 针对多种价格类型，选择价格类型时，计算费用
    //代码初审 张梦圆 20210907
    [Api(Description = "更改价格类型")]
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class PriceTypeChange : RestfulExecution<ReportItem>
    {
        public Guid ReportItemKvid { get; set; }
        public string PackageUnit { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportItemKvid.ThrowIfEmpty("未传入检测项目！");
            PackageUnit.ThrowIfNullOrEmpty("未传入PackageUnit参数！");

            var connLims = KiviiContext.GetOpenedDbConnection<Report>();

            ReportItem reportItem = null;
            #region 获取检验项目
            reportItem = connLims.SingleById<ReportItem>(ReportItemKvid);
            reportItem.ThrowIfNull("未查询到当前检测项目！");
            //2021-10-17修改此处验证，不验证项目归档，验证报告是否结算
            //if (reportItem.StatusType == ReportItemStatusType.TestCollected) throw new Exception("当前检验已归档，不可操作");
            var goodsKvid = reportItem.GoodsKvid;
            if (goodsKvid == Guid.Empty) throw new Exception("当前检测项目未关联服务无法操作！");
            var originalAmountPlan = reportItem.AmountPlan;
            #endregion

            Report report = null;
            #region 获取报告
            report = connLims.SingleById<Report>(reportItem.ReportKvid);
            report.ThrowIfNull("未查询到相关报告!");
            //若报告已结算，不能更改价格类型
            (report.AmountSettlement > 0).ThrowIfTrue("此报告已经结算，无法修改价格类型！");
            #endregion

            var connProduct = KiviiContext.GetOpenedDbConnection<Product>();
            Quotation quotation = null;
            List<QuotationDetail> quotationDetails = null;

            #region 根据报告获取报价单及报价单明细
            var quotationKvid = report.QuotationKvid;
            if (quotationKvid != Guid.Empty)
            {
                quotation = connProduct.SingleById<Quotation>(quotationKvid);
                if (quotation != null)
                {
                    var queryQuotationDetail = connProduct.From<QuotationDetail>();
                    queryQuotationDetail.Where(o => o.QuotationKvid == quotation.Kvid);
                    quotationDetails = connProduct.Select(queryQuotationDetail);
                }
            }
            #endregion
            Price price = null;

            #region 该类型的服务报价
            var queryPrice = connProduct.From<Price>();
            queryPrice.Where(o => o.GoodsKvid == reportItem.GoodsKvid && o.PackageUnit == PackageUnit);
            price = connProduct.Select(queryPrice).FirstOrDefault();
            price.ThrowIfNull("所选的价格类型无报价，请联系业务管理员！");
            #endregion

            var rtns = new RestfulUpdateResponse<ReportItem>();

            #region 处理Test,优先级，1、报价单明细，2、报价单折扣，3、服务报价
            reportItem.AmountPlan = price.Amount;
            reportItem.Amount = price.Amount;
            reportItem.PackageKvid = price.PackageKvid;
            if (quotationDetails != null)
            {
                if (quotationDetails.Exists(o => o.PriceKvid == price.Kvid))
                {
                    var quotationDetail = quotationDetails.Where(o => o.PriceKvid == price.Kvid).FirstOrDefault();

                    reportItem.Amount = quotationDetail.Amount;
                }
                else if (!quotationDetails.Exists(o => o.PriceKvid == price.Kvid) && quotation != null)
                {
                    var discount = quotation.DiscountRate;
                    if (discount <= 0 || discount > 1)
                    {
                        discount = 1;
                    }
                    reportItem.Amount = (decimal)(price.Amount * discount);
                }
                else if (!quotationDetails.Exists(o => o.PriceKvid == price.Kvid) && quotation == null)
                {
                    reportItem.Amount = price.Amount;
                }
            }
            reportItem.AddOnlyProperties(o => o.AmountPlan);
            reportItem.AddOnlyProperties(o => o.Amount);
            reportItem.AddOnlyProperties(o => o.PackageKvid);

            connLims.UpdateOnly(reportItem);
            reportItem.SetAmountLog(report, "PriceTypeChange", originalAmountPlan, reportItem.AmountPlan, connLims);
            rtns.Results = new List<ReportItem>();
            rtns.Results.Add(reportItem);
            #endregion

            return rtns;
        }
    }
    #endregion

    #region 通过选择项目模板生成检验项目
    //代码初审 张梦圆 20210907
    [Api(Description = "引用项目模板")]
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class ReportItemCreateByTemplate : RestfulExecution<ReportItem>
    {
        public Guid OwnerKvid { get; set; }
        public Guid ReportKvid { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            #region 数据验证
            if (OwnerKvid == Guid.Empty) throw new Exception("参数TemplateKvid不能为空！");
            if (ReportKvid == Guid.Empty) throw new Exception("参数ReportKvid不能为空！");
            #endregion
            var conn = KiviiContext.GetOpenedDbConnection<Template>();
            var report = conn.SingleById<Report>(ReportKvid);
            report.ThrowIfNull("未找到当前报告！无法创建项目！");

            var queryTemplateDetail = conn.From<TemplateDetail>();
            queryTemplateDetail.Where(o => o.OwnerKvid == OwnerKvid);
            var rtnTemplateDetail = conn.Select(queryTemplateDetail);
            rtnTemplateDetail.ThrowIfNullOrEmpty("所选模板无项目，无法创建检验项目");

            Quotation quotation = null;
            if (report.QuotationKvid != Guid.Empty) quotation = conn.SingleById<Quotation>(report.QuotationKvid);
            var reportItemsOperate = new List<ReportItem>();
            foreach (var en in rtnTemplateDetail)
            {
                if (en.ParentKvid != Guid.Empty) continue;
                var nodeTemplateDetails = rtnTemplateDetail.Where(o => o.ParentKvid == en.Kvid).ToList();
                var reportItem = new ReportItem();
                reportItem.PopulateWith(en);
                reportItem.Kvid = Guid.NewGuid();
                reportItem.ReportKvid = ReportKvid;
                reportItem.RootKvid = reportItem.Kvid;
                reportItem.SampleKvid = report.SampleKvid;
                reportItem.SampleRootKvid = report.SampleKvid;
                reportItem.Type = nodeTemplateDetails.Count == 0 ? ReportItemType.Test : ReportItemType.TestFolder;
                reportItem.StatusType = ReportItemStatusType.BeforeTest;
                reportItem.AmountPlan = en.Amount;
                reportItem.Amount = quotation != null ? (decimal)(en.Amount * quotation.DiscountRate) : en.Amount;
                //兼容无测试方法初始的数据
                if (reportItem.Metadata == null) reportItem.Metadata = new Dictionary<string, string>();
                if (!reportItem.Metadata.ContainsKey("DetectionCodeActual"))
                {
                    reportItem.Metadata["DetectionCodeActual"] = reportItem.DetectionCodeActual;
                }
                reportItemsOperate.Add(reportItem);
                foreach (var node in nodeTemplateDetails)
                {
                    var nodeReportItem = new ReportItem();
                    nodeReportItem.PopulateWith(node);
                    nodeReportItem.Kvid = Guid.NewGuid();
                    nodeReportItem.ReportKvid = ReportKvid;
                    nodeReportItem.ParentKvid = reportItem.Kvid;
                    nodeReportItem.RootKvid = reportItem.RootKvid;
                    nodeReportItem.SampleKvid = reportItem.SampleKvid;
                    nodeReportItem.SampleRootKvid = reportItem.SampleRootKvid;
                    nodeReportItem.Type = ReportItemType.Test;
                    nodeReportItem.StatusType = ReportItemStatusType.BeforeTest;
                    reportItemsOperate.Add(nodeReportItem);
                }
            }
            var rtns = new RestfulQueryResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();
            IDbTransaction trans = null;//define a null tag
             trans = conn.OpenTransaction();
            try
            {
                foreach (var reportItemOperate in reportItemsOperate)
                {
                    conn.Insert(reportItemOperate);
                    reportItemOperate.RemoveAllOnlyProperties();
                    rtns.Results.Add(reportItemOperate);
                    //创建标准值日志
                    reportItemOperate.SetDetectionValueQualifiedLog("create",conn,null,null, "ReportItemCreateByTemplate");
                    
                }
                trans?.Commit();
                rtns.Total = rtns.Results.Count;
                return rtns;
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }
        }
    }
    #endregion

    #region 监督报告项目统计,根据计划编号统计
    [Api(Description = "监督报告项目查询")]
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class ReportItemSuperviseQuery : RestfulExecution<ReportItem>
    {
        public string SampleSourceBatchNumber { get; set; }
        public string DemanderName { get; set; }
        public string TaskSource { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            if (SampleSourceBatchNumber.IsNullOrEmpty() && DemanderName.IsNullOrEmpty()) throw new Exception("计划编号和委托单位不能同时为空！");
            //SampleSourceBatchNumber.ThrowIfNullOrEmpty("计划编号为空，无法查询！");
            var rtns = new RestfulQueryResponse<ReportItemEx>();
            rtns.Results = new List<ReportItemEx>();

            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            //查询报告
            var queryReport = conn.From<Report>();
            if (!SampleSourceBatchNumber.IsNullOrEmpty()) queryReport.Where(o => o.SampleSourceBatchNumber == SampleSourceBatchNumber && o.Status > 700);
            if (!DemanderName.IsNullOrEmpty()) queryReport.Where(o => o.DemanderName == DemanderName);
            queryReport.OrderBy(o => o.ReportIdNumber).ThenBy(o => o.ReportId);
            var reports = conn.Select(queryReport);
            if (!TaskSource.IsNullOrEmpty())
            {
                for (var i = reports.Count - 1; i >= 0; i--)
                {
                    if (!reports[i].Metadata.ContainsKey("TaskSource") || reports[i].Metadata["TaskSource"] != TaskSource) reports.Remove(reports[i]);
                }
            }
            reports.ThrowIfNullOrEmpty("未查询到任何报告！");
            //查询检验项目
            var queryReportKvids = conn.From<Report>();
            queryReportKvids.Where(o => o.SampleSourceBatchNumber == SampleSourceBatchNumber && o.Status > 700);
            queryReportKvids.Select(o => o.Kvid);
            var queryReportItems = conn.From<ReportItem>();
            queryReportItems.Where(o => Sql.In(o.ReportKvid, queryReportKvids));
            var reportItems = conn.Select(queryReportItems);

            foreach (var report in reports)
            {
                var currentReportItems = reportItems.Where(o => o.ReportKvid == report.Kvid).ToList();
                report.SetReportItems(currentReportItems);
                var platReportItems = report.GetPlatReportItems(conn);
                foreach (var en in platReportItems)
                {
                    var reportItemEx = new ReportItemEx();
                    reportItemEx.PopulateWith(en);
                    reportItemEx.ReportId = report.ReportId;
                    try
                    {
                        reportItemEx.SamplingOrderID = report.Metadata["SamplingOrderID"];
                    }
                    catch { }
                    //合并部位信息
                    if(reportItemEx.Type == ReportItemType.TestFolder|| reportItemEx.Type == ReportItemType.Test)
                    {
                        if (reportItemEx.ParentKvid != Guid.Empty)
                        {
                            var parent = platReportItems.Where(o => o.Kvid == reportItemEx.ParentKvid).FirstOrDefault();
                            if(parent!= null&&parent.Type == ReportItemType.SamplePart)
                            {
                                reportItemEx.Title = reportItemEx.Title + "(" + parent.Title + ")";
                            }
                        }
                    }
                    if(reportItemEx.Type != ReportItemType.SamplePart)
                    {
                        rtns.Results.Add(reportItemEx);
                    }
                    //rtns.Results.Add(reportItemEx);
                }
            }
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }
    #endregion

    [Route("/Lims/ReportItem/CustomerQuery")]
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class ReportItemCustomerQuery : RestfulExecution<ReportItem>
    {
        public Guid ReportKvid { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            var rtns = new RestfulQueryResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();
            if (ReportKvid == Guid.Empty) return rtns;
            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();

            //整个报告的全部项目
            var allReportItems = conn.Select<ReportItem>(o => o.ReportKvid == ReportKvid);
            if (allReportItems.IsNullOrEmpty()) return rtns;

            #region 合并部位信息
            rtns.Results.AddRange(allReportItems);
            var sampleParts = rtns.Results.Where(o => o.Type == ReportItemType.SamplePart).ToList();
            if (sampleParts.IsNullOrEmpty())
            {
                //要是最后都没有检验项目返回了 则清除全部项目
                if (!rtns.Results.Exists(o => o.Type == ReportItemType.Test)) rtns.Results.Clear();
                rtns.Total = rtns.Results.Count;

                return rtns;
            }

            foreach (var samplePart in sampleParts)
            {
                var tests = rtns.Results.Where(o => o.ParentKvid == samplePart.Kvid).ToList();
                foreach (var test in tests)
                {
                    test.TitlePart = samplePart.Title;
                    test.ParentKvid = samplePart.ParentKvid;
                    test.PartKvid = samplePart.Kvid;
                }
            }

            rtns.Results.RemoveAll(o => sampleParts.ConvertAll(p => p.Kvid).Contains(o.Kvid));
            #endregion

            //要是最后都没有检验项目返回了 则清除全部项目
            if (!rtns.Results.Exists(o => o.Type == ReportItemType.Test || o.Type == ReportItemType.TestFolder)) rtns.Results.Clear();

            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }

    #region 匹配标准值(升级)
    [Api(Description = "获取标准值(多条件)")]
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class ReportItemGetDetectionQualifiedValue : RestfulExecution<ReportItem>
    {
        public List<ReportItem> ReportItems { get; set; }
        public string SampleGrade { get; set; }
        public string SampleGradeSafety { get; set; }
        public string SampleGradeFunction { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            #region 数据验证
            //(SampleGrade.IsNullOrEmpty() && SampleGradeSafety.IsNullOrEmpty()).ThrowIfTrue("未传入样品等级或安全类别!");
            (ReportItems == null || ReportItems.Count == 0).ThrowIfTrue("未传入检验项目集合!");
            #endregion
            var rtns = new RestfulQueryResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();
            var connLims = KiviiContext.GetOpenedDbConnection<Report>();
            var connStds = KiviiContext.GetOpenedDbConnection<DetectionValue>();
            #region 筛选检验项目,只保留未维护标准值的数据(状态+标准值+类型)
            //获取报告信息
            var report = connLims.SingleById<Report>(ReportItems[0].ReportKvid);
            if (report == null) throw new Exception("未获取报告信息");
            if (report.StatusType != ReportStatusType.CommissionAccept && report.StatusType != ReportStatusType.DataEntry) throw new Exception("仅允许操作委托受理或数据录入的报告!");
            var reportItemsAll = connLims.Select<ReportItem>(o => o.ReportKvid == report.Kvid);
            //设置状态范围
            var statusScope = new List<int>();
            if (report.StatusType == ReportStatusType.CommissionAccept)
            {
                statusScope.Add((int)ReportItemStatusType.BeforeTest);
            }
            else
            {
                //存在报告状态为委托受理的情况,但项目已经到达录入
                //该情况仅允许修改状态值为200~300之间的数据
                statusScope.Add((int)ReportItemStatusType.Testing);
                statusScope.Add((int)ReportItemStatusType.BeforeReview);
                statusScope.Add((int)ReportItemStatusType.TestFinished);
            }
            var reportItems = ReportItems.Where(o => statusScope.Contains(o.Status) && o.Type == ReportItemType.Test).ToList();
            //过滤可直接匹配标准值的检验项目
            //获取所有检验项目和标准值集合
            var detectionKvids = reportItems.ConvertAll(o => o.DetectionKvid).Distinct().ToList();
            //获取检验项目(Detection)集合
            var detections = connStds.Select<Detection>(o => Sql.In(o.Kvid, detectionKvids));
            //获取标准值集合
            var detectionValues = connStds.Select<DetectionValue>(o => Sql.In(o.DetectionKvid, detectionKvids));
            var reportItemsOperate = new List<ReportItem>();
            foreach (var reportItem in reportItems)
            {
                var detection = detections.FirstOrDefault(o => o.Kvid == reportItem.DetectionKvid);
                if (detection == null) continue;
                var detectionQualifiedValue = detection.GetQualifiedValueUpgrade(SampleGrade, SampleGradeSafety,SampleGradeFunction, detectionValues);
                if (detectionQualifiedValue != "请选择对应标准值!") continue;
                reportItemsOperate.Add(reportItem);
            }
            reportItems = reportItemsOperate;
            detectionKvids = reportItems.ConvertAll(o => o.DetectionKvid).Distinct().ToList();
            detectionValues = detectionValues.Where(o => Sql.In(o.DetectionKvid, detectionKvids)).ToList();
            if (reportItems.Count == 0) return rtns;
            #endregion
            if (detectionValues.Count == 0) return rtns;
            //标准值分类:优一合+ABC类-子项；非优一合+ABC类-主项；非优一合+ABC-子项
            var detectionValuesParent = detectionValues.Where(o => o.ParentKvid == Guid.Empty).ToList();
            #region 如果参数中样品等级、安全类别不为空，过滤数据
            var SampleGradeListRemove = new List<string>();
            var SampleGradeListSave=new List<string>();
            if (SampleGradeFunction.IsNullOrEmpty())
            {
                if (!SampleGrade.IsNullOrEmpty())
                {
                    switch (SampleGrade)
                    {
                        case "优等品":
                            SampleGradeListRemove.Add("一等品");
                            SampleGradeListRemove.Add("二等品");
                            SampleGradeListRemove.Add("合格品");
                            break;
                        case "一等品":
                            SampleGradeListRemove.Add("优等品");
                            SampleGradeListRemove.Add("二等品");
                            SampleGradeListRemove.Add("合格品");
                            break;
                        case "二等品":
                            SampleGradeListRemove.Add("优等品");
                            SampleGradeListRemove.Add("一等品");
                            SampleGradeListRemove.Add("合格品");
                            break;
                        case "合格品":
                            SampleGradeListRemove.Add("优等品");
                            SampleGradeListRemove.Add("一等品");
                            SampleGradeListRemove.Add("二等品");
                            break;
                        default:
                            break;

                    }
                }
                if (!SampleGradeSafety.IsNullOrEmpty())
                {
                    switch (SampleGradeSafety)
                    {
                        case "A类":
                            SampleGradeListRemove.Add("B类");
                            SampleGradeListRemove.Add("C类");
                            break;
                        case "B类":
                            SampleGradeListRemove.Add("A类");
                            SampleGradeListRemove.Add("C类");
                            break;
                        case "C类":
                            SampleGradeListRemove.Add("A类");
                            SampleGradeListRemove.Add("B类");
                            break;
                        default:
                            break;
                    }
                }
                if (SampleGradeListRemove.Count > 0) detectionValuesParent = detectionValuesParent.Where(o => !SampleGradeListRemove.Contains(o.Condition)).ToList();
            }
            else {
                SampleGradeListSave.Add(SampleGradeFunction);
                if (!SampleGrade.IsNullOrEmpty())
                {
                    SampleGradeListSave.Add(SampleGrade);
                }
                else {
                    SampleGradeListSave.Add("优等品");
                    SampleGradeListSave.Add("一等品");
                    SampleGradeListSave.Add("二等品");
                    SampleGradeListSave.Add("合格品");
                }
                if (!SampleGradeSafety.IsNullOrEmpty())
                {
                    SampleGradeListSave.Add(SampleGradeSafety);
                }
                else {
                    SampleGradeListSave.Add("A类");
                    SampleGradeListSave.Add("B类");
                    SampleGradeListSave.Add("C类");
                }
                if (SampleGradeListSave.Count > 0) detectionValuesParent = detectionValuesParent.Where(o => SampleGradeListSave.Contains(o.Condition)).ToList();
            }
            if (detectionValuesParent.Count == 0) return rtns;
            var detectionValuesParentKvid = detectionValuesParent.ConvertAll(o => o.Kvid);
            detectionValues = detectionValues.Where(o => Sql.In(o.Kvid, detectionValuesParentKvid) || Sql.In(o.ParentKvid, detectionValuesParentKvid)).OrderByDescending(o => o.ParentKvid).ToList();
            #endregion
            foreach (var reportItem in reportItems)
            {
                //组织项目名称信息
                //获取是否存在部位信息,不可直接通过页面组织的QueryEx类数据直接获取部位,如果用户当前仅选中子项,获取失败
                var title = reportItem.Title;
                if (!reportItem.TitlePart.IsNullOrEmpty())
                {
                    title = $"{title}({reportItem.TitlePart})";
                }
                else
                {
                    if (reportItem.ParentKvid != Guid.Empty)
                    {
                        var reportItemParent = reportItemsAll.SingleOrDefault(o => o.Kvid == reportItem.ParentKvid);
                        if (reportItemParent != null && reportItemParent.Type != ReportItemType.SampleSelf)
                        {
                            if (reportItemParent.Type == ReportItemType.SamplePart)
                            {
                                title = $"{reportItemParent.Title}({title})";
                            }
                            else
                            {
                                title = $"{reportItemParent.Title}-{title}";
                                if (reportItemParent.ParentKvid != Guid.Empty)
                                {
                                    var reportItemPart = reportItemsAll.SingleOrDefault(o => o.Kvid == reportItemParent.ParentKvid && o.Type == ReportItemType.SamplePart);
                                    if (reportItemPart != null) title = $"{title}({reportItemPart.Title})";
                                }
                            }
                        }
                    }
                }
                //获取相关标准值信息
                var currentDeteValues = detectionValues.Where(o => o.DetectionKvid == reportItem.DetectionKvid).ToList();
                var deteValueKvidsProcessed = new List<Guid>();
                foreach (var currentDeteValue in currentDeteValues)
                {
                    if (deteValueKvidsProcessed.Exists(o => o == currentDeteValue.Kvid)) continue;
                    var condition = currentDeteValue.Condition;
                    if (currentDeteValue.ParentKvid != Guid.Empty)
                    {
                        //如果为非优一合条件（继续细分不同条件）,需组合条件
                        deteValueKvidsProcessed.Add(currentDeteValue.ParentKvid);
                        var deteValueParent = currentDeteValues.SingleOrDefault(o => o.Kvid == currentDeteValue.ParentKvid);
                        if (deteValueParent != null && deteValueParent.Condition != SampleGrade && deteValueParent.Condition != SampleGradeSafety)
                        {
                            var conditionType = deteValueParent.Condition.Replace("（", "").Replace("）", "").Trim();
                            condition = $"{condition}({conditionType})";
                        }
                    }
                    var reportItemWithDetectionQualifiedValue = new ReportItem();
                    reportItemWithDetectionQualifiedValue.ReportKvid = reportItem.ReportKvid;
                    reportItemWithDetectionQualifiedValue.Title = title;
                    reportItemWithDetectionQualifiedValue.DetectionTitle = condition;
                    reportItemWithDetectionQualifiedValue.DetectionValueQualified = currentDeteValue.QualifiedValue;
                    //用DetectionUnit记录默认条件值
                    if (reportItem.Metadata == null)
                    {
                        reportItemWithDetectionQualifiedValue.DetectionUnit = string.Empty;
                    }
                    else
                    {
                        reportItemWithDetectionQualifiedValue.DetectionUnit = reportItem.Metadata.ContainsKey("DetectionValueCondition") ? reportItem.Metadata["DetectionValueCondition"] : string.Empty;
                    }
                    reportItemWithDetectionQualifiedValue.SortId = reportItem.SortId;
                    reportItemWithDetectionQualifiedValue.Status = reportItem.Status;
                    reportItemWithDetectionQualifiedValue.Type = reportItem.Type;
                    reportItemWithDetectionQualifiedValue.Kvid = reportItem.Kvid;
                    rtns.Results.Add(reportItemWithDetectionQualifiedValue);
                    deteValueKvidsProcessed.Add(currentDeteValue.Kvid);
                }
                rtns.Results = rtns.Results.OrderBy(o => o.SortId).ToList();
            }
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }

    [Api(Description = "匹配标准值(多项)")]
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class ReportItemMatchDetectionQualifiedValue : RestfulExecution<ReportItem>
    {
        public List<ReportItem> ReportItems { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            #region 数据验证
            (ReportItems == null || ReportItems.Count == 0).ThrowIfTrue("未传入检验项目信息!");
            #endregion
            var rtns = new RestfulQueryResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();
            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            //获取报告信息
            var report = conn.SingleById<Report>(ReportItems[0].ReportKvid);
            if (report == null) throw new Exception("未获取报告信息");
            if (report.StatusType != ReportStatusType.CommissionAccept && report.StatusType != ReportStatusType.DataEntry) throw new Exception("仅允许操作委托受理或数据录入的报告!");
            //设置状态范围
            var statusScope = new List<int>();
            if (report.StatusType == ReportStatusType.CommissionAccept)
            {
                statusScope.Add((int)ReportItemStatusType.BeforeTest);
            }
            else
            {
                //存在报告状态为委托受理的情况,但项目已经到达录入
                //该情况仅允许修改状态值为200~300之间的数据
                statusScope.Add((int)ReportItemStatusType.Testing);
                statusScope.Add((int)ReportItemStatusType.BeforeReview);
                statusScope.Add((int)ReportItemStatusType.TestFinished);
            }
            var reportItems = ReportItems.Where(o => statusScope.Contains(o.Status) && o.Type == ReportItemType.Test).ToList();
            var reportItemsAll = conn.Select<ReportItem>(o => o.ReportKvid == ReportItems[0].ReportKvid);
            if (reportItems.Count == 0) return rtns;
            foreach (var reportItem in reportItems)
            {
                //获取更新标准值前项目信息
                var reportItemAll = reportItemsAll.FirstOrDefault(o => o.Kvid == reportItem.Kvid);
                reportItem.RemoveAllOnlyProperties();
                reportItem.AddOnlyProperties(o => o.DetectionValueQualified);
                //传参时已经将条件值传入Metadata中,更新即可
                reportItem.AddOnlyProperties(o => o.Metadata);
                conn.UpdateOnly(reportItem);
                reportItem.RemoveAllOnlyProperties();
                rtns.Results.Add(reportItem);
            }
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }
    #endregion

    #region 标准值比对
    [Api(Description = "标准值比对筛选")]
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class ReportItemGetStandards : RestfulExecution<Standard>
    {
        public List<ReportItem> ReportItems { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            #region 数据验证
            if (ReportItems.Count == 0) throw new Exception("未传入检验项目!");
            #endregion
            var rtns=new RestfulQueryResponse<Standard>();
            rtns.Results = new List<Standard>();
            var detectionReferenceKvids = ReportItems.ConvertAll(o=>o.DetectionReferenceKvid);
            var connStds = KiviiContext.GetOpenedDbConnection<Standard>();
            var standardKvids = connStds.Select<Detection>(o => o.ReferenceDetectionKvid!=Guid.Empty&&Sql.In(o.ReferenceDetectionKvid, detectionReferenceKvids)).Select(o => o.StandardKvid).Distinct().ToList();
            var standards = connStds.Select<Standard>(o => Sql.In(o.Kvid, standardKvids)).ToList();
            rtns.Results.AddRange(standards);
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }

    [Api(Description = "标准值比对结果")]
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class ReportItemGetDetectionValueQualifiedDiff : RestfulExecution<ReportItem>
    { 
        public List<ReportItem> ReportItems { get; set; }
        public List<Standard> StandardsList { get; set; }
        public string SampleGrade { get; set; }
        public string SampleGradeSafety { get; set; }
        public string SampleGradeFunction { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            #region 数据验证
            if (ReportItems.Count == 0) throw new Exception("未传入检验项目!");
            if (StandardsList.Count == 0) throw new Exception("未传入比对标准!");
            if (SampleGrade.IsNullOrEmpty() && SampleGradeSafety.IsNullOrEmpty() && SampleGradeFunction.IsNullOrEmpty()) throw new Exception("样品等级、安全类别、功能等级皆为空，禁止比对！");
            if (ReportItems.Exists(o => o.Status != (int)ReportItemStatusType.BeforeTest)) throw new Exception("仅允许更新待检测状态的数据!");
            var parentKvidsByChildren = ReportItems.Where(o => o.ParentKvid != Guid.Empty).ToList().ConvertAll(o => o.ParentKvid).Distinct().ToList();
            var parentKvids = ReportItems.Where(o => o.ParentKvid == Guid.Empty).ToList().ConvertAll(o=>o.Kvid);
            if (parentKvidsByChildren.Except(parentKvids).ToList().Count > 0) throw new Exception("存在单独子项,请重新选择!");
            #endregion
            //获取涉及检验项目及标准值数据
            var connStds = KiviiContext.GetOpenedDbConnection<Standard>();
            var standardKvids = StandardsList.ConvertAll(o=>o.Kvid);
            var detectionReferenceKvidsParent = ReportItems.Where(o=>o.DetectionReferenceKvid!=Guid.Empty).ToList().ConvertAll(o=>o.DetectionReferenceKvid);
            var detectionsAll = new List<Detection>();
            var detectionParentsRefs=connStds.Select<Detection>(o => Sql.In(o.StandardKvid, standardKvids) &&Sql.In(o.ReferenceDetectionKvid, detectionReferenceKvidsParent)).ToList();
            detectionsAll.AddRange(detectionParentsRefs);
            var detectionParentsRefsKvids = detectionParentsRefs.ConvertAll(o=>o.Kvid);
            var detectionsChildrenRefs = connStds.Select<Detection>(o => Sql.In(o.ParentKvid, detectionParentsRefsKvids)).ToList();
            detectionsAll.AddRange(detectionsChildrenRefs);
            var detectionKvids = detectionsAll.ConvertAll(o=>o.Kvid);
            var queryDetectionValues = connStds.From<DetectionValue>();
            queryDetectionValues.Where(o => Sql.In(o.DetectionKvid, detectionKvids));
            var detectionValues = connStds.Select(queryDetectionValues);
            //组织返回数据
            var reportItemsParent = ReportItems.Where(o=>o.ParentKvid==Guid.Empty).OrderBy(o=>o.SortId).ToList();
            var rtns = new RestfulQueryResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();
            foreach (var reportItemParent in reportItemsParent)
            {
                foreach (var standard in StandardsList)
                {
                    var reportItemParentRtns = new ReportItem();
                    reportItemParentRtns.PopulateWith(reportItemParent);
                    reportItemParentRtns.StandardKvid= standard.Kvid;
                    reportItemParentRtns.StandardCode = standard.Code;
                    reportItemParentRtns.StandardTitle = standard.Title;
                    //耐干摩和耐湿磨需特殊处理
                    var detectionParent = new Detection();
                    if (reportItemParent.DetectionTitle.Contains("耐干摩擦色牢度"))
                    {
                        detectionParent = detectionsAll.FirstOrDefault(o => o.ReferenceDetectionKvid == reportItemParent.DetectionReferenceKvid && o.StandardKvid == standard.Kvid&&o.Title.Contains("耐干摩擦色牢度"));
                    }
                    else if (reportItemParent.DetectionTitle.Contains("耐湿摩擦色牢度"))
                    {
                        detectionParent = detectionsAll.FirstOrDefault(o => o.ReferenceDetectionKvid == reportItemParent.DetectionReferenceKvid && o.StandardKvid == standard.Kvid && o.Title.Contains("耐湿摩擦色牢度"));
                    }
                    else {
                        detectionParent = detectionsAll.FirstOrDefault(o => o.ReferenceDetectionKvid == reportItemParent.DetectionReferenceKvid && o.StandardKvid == standard.Kvid&& !o.Title.Contains("耐干摩擦色牢度") && !o.Title.Contains("耐湿摩擦色牢度"));
                    }
                    reportItemParentRtns.DetectionKvid = detectionParent == null?Guid.Empty:detectionParent.Kvid;
                    reportItemParentRtns.DetectionReferenceKvid = detectionParent == null ? Guid.Empty:detectionParent.ReferenceDetectionKvid;
                    reportItemParentRtns.DetectionTitle = detectionParent == null ? string.Empty:detectionParent.Title;
                    reportItemParentRtns.DetectionCodeActual = detectionParent == null ? string.Empty:detectionParent.CodeActual;
                    reportItemParentRtns.DetectionValueQualified = detectionParent == null ? "无匹配标准值":detectionParent.GetQualifiedValueUpgrade(SampleGrade, SampleGradeSafety, SampleGradeFunction,detectionValues);
                    reportItemParentRtns.RemoveAllOnlyProperties();

                    rtns.Results.Add(reportItemParentRtns);
                    var reportItemsChildren = ReportItems.Where(o => o.ParentKvid == reportItemParent.Kvid).OrderBy(o => o.SortId).ToList();
                    var detectionsChildren = new List<Detection>();
                    if (detectionParent != null) {
                        detectionsChildren.AddRange(detectionsAll.Where(o => o.ParentKvid == detectionParent.Kvid).OrderBy(o => o.SortId).ToList());
                    }
                    for (int i = 0; i < reportItemsChildren.Count; i++)
                    { 
                        var reportItemChild= reportItemsChildren[i];
                        var reportItemChildRtns = new ReportItem();
                        reportItemChildRtns.PopulateWith(reportItemChild);
                        reportItemChildRtns.StandardKvid = standard.Kvid;
                        reportItemChildRtns.DetectionKvid = detectionsChildren.Count==0?Guid.Empty:detectionsChildren[i].Kvid;
                        var detectionValueQualified = string.Empty;
                        if (reportItemsChildren.Count != detectionsChildren.Count)
                        {
                            detectionValueQualified = "子项不一致,匹配失败";
                        }
                        else 
                        {
                            if (detectionsChildren.Count == 0)
                            {
                                detectionValueQualified = "无匹配标准值";
                            }
                            else {
                                detectionValueQualified = detectionsChildren[i].GetQualifiedValueUpgrade(SampleGrade, SampleGradeSafety, SampleGradeFunction, detectionValues);
                            }  
                        }
                        reportItemChildRtns.DetectionValueQualified= detectionValueQualified;
                        reportItemChildRtns.RemoveAllOnlyProperties();
                        rtns.Results.Add(reportItemChildRtns);
                    }
                }
            }
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }
    #endregion
}
